﻿/**
* DevExpress Dashboard (dashboard-control.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { _addLicenseTrigger, _setShowMessageFunc, appendStaticContextToRootViewModel, useKoIntegration } from '@devexpress/analytics-core/analytics-internal';
import { addToBindingsCache } from '@devexpress/analytics-core/analytics-widgets-internal';
import { EventsStrategy } from 'devextreme/core/events_strategy';
import { Options } from 'devextreme/core/options';
import dxPopup from 'devextreme/ui/popup';
import * as $ from 'jquery';
import * as ko from 'knockout';
import { $unwrap, createJQueryCallbacks, createJQueryDeferred, deepExtend, extend, getHeight } from '../data/_jquery-helpers';
import { groupByProperty, type } from '../data/_utils';
import { getLocalizationById } from '../data/localization/_default';
import { Dashboard } from '../model/dashboard';
import { safeComputed, subscribeArrayChange } from '../model/internal/_knockout-utils';
import { defineObsoleteMethod, defineObsoleteProperty } from '../model/internal/_obsolete-helper';
import { CustomItem } from '../model/items/custom-item/custom-item';
import { itemTypesMap } from '../model/serializable-model';
import { _initializeHtmlTemplates } from '../resources/_html_templates.generated';
import { _initializeSvgTemplates } from '../resources/_svg_templates.generated';
import { getBaseColorScheme } from '../viewer-parts/_dx-devextreme-themes-integration';
import { LegacySettings } from '../viewer-parts/legacy-settings';
import { createStylesElement } from '../viewer-parts/viewer/_cssHelper';
import { DataSourceBrowser } from './_data-source-browser';
import { CancellationToken, combineOptions, requestParamsValidator, toCancelableSequenceAction } from './_helpers';
import { _registerKoComponents } from './_ko-components-helper';
import { DesignerDataServiceClient, ViewerDataServiceClient } from './_service-client';
import { defaultExtensions, designerExtensions, extensionNameMap } from './control-options';
import { DashboardUpdateHub } from './dashboard-update-hub/_dashboard-update-hub';
import { DockingLayoutAdapter } from './docking-layout/_docking-layout-adapter';
import { SingleItemSizeController } from './internal/_interfaces';
import { getOptionNameByEvent } from './internal/_options-manager';
import { UpdateController } from './internal/_update-controller';
import { NotificationController } from './notification-controller/notificator';
import { AjaxRemoteService, FetchRemoteService } from './remote-service';
import { ResourceManager } from './resource-manager';
import { ViewerApi } from './viewer/_viewer-api';
import { ViewerItemAdaptersManager } from './viewer/adapters/_item-viewer-adapter-factory';
export class DashboardControl {
    static _recursiveAsyncEval(sequence, cancellationToken, previousOptions, currentIndex = 0) {
        if (!previousOptions) {
            previousOptions = {
                surfaceLeft: 0
            };
        }
        var def = createJQueryDeferred();
        let sequenceActions = sequence[currentIndex];
        if (sequenceActions) {
            let actions = sequenceActions.map(sa => sa.cancelableAction);
            $.when(...actions.map(a => a(previousOptions, cancellationToken)))
                .done((...options) => {
                if (cancellationToken.canceled)
                    def.reject();
                else {
                    DashboardControl._recursiveAsyncEval(sequence, cancellationToken, combineOptions(previousOptions, options), currentIndex + 1)
                        .done((opts) => {
                        if (cancellationToken.canceled)
                            def.reject();
                        else
                            def.resolve(opts);
                    })
                        .fail(() => def.reject());
                }
            })
                .fail(() => def.reject());
        }
        else {
            if (cancellationToken.canceled)
                def.reject();
            else {
                if (cancellationToken.canceled)
                    def.reject();
                else
                    def.resolve(previousOptions);
            }
        }
        return def.promise();
    }
    get _updateHub() {
        return this._updateHubPrivate;
    }
    get encodeHtml() {
        return this.option('encodeHtml');
    }
    get showConfirmationOnBrowserClosing() { return this.option('showConfirmationOnBrowserClosing'); }
    get allowSwitchToDesigner() {
        return this._allowSwitchToDesigner;
    }
    get maximizedDashboardItemName() {
        var fullscreenItemProvider = this._getFullscreenItemProvider();
        return fullscreenItemProvider && fullscreenItemProvider.maximizedItemName || '';
    }
    get extensions() {
        return this._extensions();
    }
    get _classList() {
        if (this._surfaceTop() !== 0) {
            return 'dx-dashboard-surface-toolbar-visible';
        }
    }
    element() {
        return this._element;
    }
    on(eventName, eventHandler) {
        this._eventsStrategy.on(eventName, eventHandler);
    }
    off(eventName, eventHandler) {
        this._eventsStrategy.off(eventName, eventHandler);
    }
    _raiseEvent(eventName, eventArgs) {
        this._eventsStrategy.fireEvent(eventName, [eventArgs]);
        var optionName = getOptionNameByEvent(eventName);
        if (optionName) {
            let func = this.option(optionName);
            if (func) {
                func.call(this, eventArgs);
            }
        }
    }
    option(...args) {
        return this._options.option(...args);
    }
    _silent(...args) {
        return this._options.silent(...args);
    }
    resetOption(name) {
        this.beginUpdate();
        this._options.reset(name);
        this.endUpdate();
    }
    beginUpdate() {
        this._updateController.beginUpdate();
    }
    endUpdate() {
        this._updateController.endUpdate();
    }
    _getOptionValue(name, context) {
        const value = this.option(name);
        return value;
    }
    _getDefaultOptions() {
        return {
            limitVisibleDataMode: 'Designer',
            resizeByTimer: true,
            encodeHtml: true,
            useNeutralFilterMode: true,
            allowMaximizeItems: true,
            useCardLegacyLayout: false,
            showConfirmationOnBrowserClosing: true,
            workingMode: 'Designer',
            loadDefaultDashboard: true,
            dataRequestOptions: {},
            extensions: {}
        };
    }
    _getOptionsByReference() {
        return {};
    }
    _getDeprecatedOptions() {
        return {};
    }
    _notifyOptionChanged(option, value) {
        document.createElement;
        const name = option;
        const args = {
            name: name.split(/[.[]/)[0],
            fullName: name,
            value: value,
            component: this
        };
        this._optionChanged(args);
        this._raiseEvent('optionChanged', args);
    }
    _reinitializeDashboard() {
        if (this.dashboard()) {
            var dashboardState = this.getDashboardState();
            this.initializeDashboard(this.option('dashboardId'), this.dashboard().getJSON(), dashboardState && JSON.parse(this.getDashboardState()) || '');
        }
    }
    _getCebabCaseExtensionName(extensionName) {
        const index = Object.keys(extensionNameMap).map(k => extensionNameMap[k]).indexOf(extensionName);
        return index === -1 ? extensionName : Object.keys(extensionNameMap)[index];
    }
    _getCamelCaseExtensionName(extensionName) {
        return extensionNameMap[extensionName] || extensionName;
    }
    _optionChanging(name, value, previousValue) {
        switch (name) {
            case 'workingMode':
                if (this._updateController.getControlState() !== 'controlInitializing' && (previousValue === 'ViewerOnly' || value === 'ViewerOnly')) {
                    throw new Error(`Cannot switch to ${value} from ${previousValue} mode.`);
                }
                break;
        }
    }
    _optionChanged(args) {
        switch (args.name) {
            case 'dashboardId':
                if (this._updateController.getControlState() !== 'dashboardInitializing')
                    this._updateController.addAction('loadDashboard');
                break;
            case 'workingMode':
                if (this._updateController.getControlState() === 'controlInitializing') {
                    this.isDesignMode(args.value === 'Designer');
                }
                else {
                    if (args.value === 'Designer') {
                        this._switchToDesigner();
                    }
                    else {
                        this._switchToViewer();
                    }
                }
                break;
            case 'initialDashboardState':
                break;
            case 'useNeutralFilterMode':
                this._updateController.addAction('reinitializeDashboard');
                break;
            case 'useCardLegacyLayout':
                break;
            case 'resizeByTimer':
                this.resizeByTimer(args.value);
                break;
            case 'endpoint':
                break;
            case 'dataRequestOptions':
                this._updateController.addAction('reinitializeDashboard');
                break;
            case 'encodeHtml':
                this._updateController.addAction('reinitializeDashboard');
                break;
            case 'ajaxRemoteService':
            case 'fetchRemoteService':
                this._updateRemoteServiceOptions(args.name);
                break;
            case 'limitVisibleDataMode':
                this._updateController.addAction('reinitializeDashboard');
                break;
            case 'onInitializing':
            case 'onOptionChanged':
                break;
            case 'allowMaximizeItems':
                if (this._actualLayoutController()) {
                    this._actualLayoutController().allowMaximizeItems = args.value;
                }
                this._updateController.addAction('updateItemToolbars');
                break;
            case 'extensions':
                if (this._updateController.getControlState() !== 'controlInitializing') {
                    this._processExtensionsOption(args);
                }
                break;
            case 'onDashboardInitializing':
            case 'onDashboardInitialized':
            case 'onDashboardStateChanged':
            case 'onItemBeginUpdate':
            case 'onItemEndUpdate':
            case 'onDashboardBeginUpdate':
            case 'onDashboardEndUpdate':
            case 'showConfirmationOnBrowserClosing':
            case 'loadDefaultDashboard':
                break;
        }
        this._updateController.commitUpdate();
    }
    _updateRemoteServiceOptions(remoteServiceOptionsKey) {
        const remoteServiceOptions = this.option(remoteServiceOptionsKey);
        const remoteServiceType = remoteServiceOptionsKey === 'ajaxRemoteService' ? AjaxRemoteService : FetchRemoteService;
        const controlState = this._updateController.getControlState();
        const canChangeRemoteService = controlState === 'controlInitializing' || controlState === 'controlInitialized';
        if (this.remoteService && this.remoteService instanceof remoteServiceType) {
            if (canChangeRemoteService && !remoteServiceOptions) {
                this.remoteService = null;
            }
            else {
                this.remoteService._applyOptions(remoteServiceOptions);
            }
        }
        else if (canChangeRemoteService) {
            this.remoteService = this._remoteServiceFactory();
        }
    }
    _processExtensionsOption(args) {
        if (args.name === args.fullName && args.value === false) {
            this._unregisterExtensionCore(...this._extensions().map(extension => extension.name));
            return;
        }
        if (args.name === args.fullName && typeof args.value === 'object') {
            this._processExtensionsOptions(args.value);
            return;
        }
        else {
            const parts = args.fullName
                .split(/(?:\[|\]|\.)/)
                .filter(item => !!item.length);
            const extensionName = parts[1];
            let extensionOptionParts = parts.slice(2);
            this._processExtensionsOptions({
                [extensionName]: this._createExtensionOption(extensionOptionParts, args.value)
            });
        }
    }
    _createExtensionOption(parts, value) {
        if (parts.length === 0)
            return value;
        return parts.slice(0, parts.length - 1).reverse().reduce((acc, cur) => {
            return { [cur]: acc };
        }, {
            [parts[parts.length - 1]]: value
        });
    }
    _processExtensionsOptions(extensionsOptions) {
        Object.keys(extensionsOptions).forEach(key => {
            let extensionName = key;
            let extensionOptions = extensionsOptions[extensionName];
            if (extensionOptions === false) {
                this._unregisterExtensionCore(extensionName);
            }
            else if (typeof extensionOptions === 'object') {
                var extension = this.findExtension(extensionName);
                if (!extension) {
                    const extensionsFactory = { ...defaultExtensions, ...designerExtensions };
                    const extensionCreator = extensionsFactory[this._getCebabCaseExtensionName(extensionName)];
                    type.isFunction(extensionCreator) && this.registerExtension(extensionCreator(this, extensionOptions));
                }
                else {
                    var optionsManager = extension && extension._optionsManager;
                    if (optionsManager) {
                        Object.keys(extensionOptions).forEach((optionName) => {
                            const extensionArgs = { name: optionName, fullName: optionName, value: extensionOptions[optionName] };
                            var extensionAction = optionsManager.optionChanged(extensionArgs);
                            if (extensionAction)
                                this._updateController.addAction(extensionAction);
                        });
                    }
                }
            }
        });
    }
    constructor(element, options = {}) {
        this._repaintRequest = createJQueryCallbacks();
        this._extensions = ko.observableArray();
        this._serviceClient = ko.observable();
        this._displayAllData = ko.observable();
        this._isLoading = ko.observable(false);
        this._perDashboardDisposables = [];
        this._layoutInfo = ko.observable(null);
        this._layoutInfoProviderCollection = [];
        this._actualLayoutController = ko.pureComputed(() => this._layoutInfo() && this._layoutInfo().layoutController());
        this._emptyControlTemplates = ko.observableArray();
        this._surfaceLeft = ko.observable(0);
        this._surfaceTop = ko.observable(0);
        this.surfaceLeft = ko.observable(0);
        this.surfaceTop = ko.observable(0);
        this.isDesignMode = ko.observable(true);
        this.dashboardContainer = ko.observable();
        this.dashboard = ko.computed(() => this.dashboardContainer() && this.dashboardContainer().dashboard || null);
        this.customTemplates = ko.observableArray();
        this.getWidgetContainer = () => {
            if (!this._widgetContainer) {
                let wc = this._element.querySelector('.dx-dashboard-widget-container');
                if (wc) {
                    this._widgetContainer = wc;
                }
            }
            return this._widgetContainer;
        };
        this._updateController = new UpdateController({
            'updateItemToolbars': {
                action: () => {
                    if (this.dashboard()) {
                        this._viewerApi.updateItemCaptionToolbar();
                    }
                },
                masterActions: ['refreshData', 'reinitializeDashboard', 'loadDashboard']
            },
            'updateDashboardToolbar': {
                action: () => this._reinitializeDashboard(),
                masterActions: ['reinitializeDashboard', 'loadDashboard']
            },
            'refreshData': {
                action: () => this._reinitializeDashboard(),
                masterActions: ['reinitializeDashboard', 'loadDashboard']
            },
            'reinitializeDashboard': {
                action: () => this._reinitializeDashboard(),
                masterActions: ['loadDashboard']
            },
            'loadDashboard': {
                action: () => {
                    if (this.option('dashboardId')) {
                        this._loadDashboardCore(this.option('dashboardId'));
                    }
                    else {
                        this._unloadDashboardCore();
                    }
                }
            },
            'noop': {
                action: () => { },
                masterActions: ['updateItemToolbars', 'updateDashboardToolbar', 'refreshData', 'reinitializeDashboard', 'loadDashboard'],
            }
        }, (stateProvider, action) => {
            switch (stateProvider()) {
                case 'controlInitializing':
                    return 'noop';
                case 'controlInitialized':
                case 'dashboardLoading':
                case 'dashboardLoaded':
                case 'dashboardInitializing':
                    return action === 'loadDashboard' ? action : 'noop';
                case 'dashboardInitialized':
                    return action;
            }
        });
        this._workingModeChangeDeferred = createJQueryDeferred();
        _addLicenseTrigger(options._licenseOptions);
        useKoIntegration();
        _initializeHtmlTemplates();
        _initializeSvgTemplates();
        _registerKoComponents();
        createStylesElement(element, options.nonce);
        this._updateController.switchControlState('controlInitializing');
        this._element = element;
        this._controlOptions = this._getDefaultOptions();
        this._defaultControlOptions = this._getDefaultOptions();
        this._eventsStrategy = EventsStrategy.create(this, options.eventsStrategy);
        this._options = new Options(this._controlOptions, this._defaultControlOptions, this._getOptionsByReference(), this._getDeprecatedOptions());
        this._options.onChanging((name, previousValue, value) => this._optionChanging(name, value, previousValue));
        this._options.onChanged((name, value, previousValue) => this._notifyOptionChanged(name, value));
        this._options.onStartChange(() => this.beginUpdate());
        this._options.onEndChange(() => this.endUpdate());
        if (options.extensions) {
            for (let extensionName in options.extensions) {
                if (extensionNameMap.hasOwnProperty(extensionName)) {
                    const newName = extensionNameMap[extensionName];
                    if (newName !== extensionName) {
                        options.extensions[newName] = options.extensions[extensionName];
                        delete options.extensions[extensionName];
                    }
                }
            }
        }
        this.resizeByTimer = ko.observable(this._defaultControlOptions.resizeByTimer);
        this.option(options);
        this._endpointCollection = this._unrollEndpointCollection(this.option());
        this.remoteService = this._remoteServiceFactory();
        LegacySettings._useCardLegacyLayout = this.option('useCardLegacyLayout');
        this.surfaceLeft.subscribe(value => {
            this._surfaceLeft(value);
            this.repaint();
        });
        this.surfaceTop.subscribe(value => {
            this._surfaceTop(value);
            this.repaint();
        });
        this._dashboardContext = {
            beforeApplyViewerItemOptions: createJQueryCallbacks(),
            viewerItemCreated: createJQueryCallbacks(),
            viewerItemDispose: createJQueryCallbacks(),
            addContextToolbarItems: createJQueryCallbacks(),
            viewerItemCreator: {},
            refresh: (itemName) => this.refresh(itemName),
            useNeutralFilterMode: () => this.option('useNeutralFilterMode'),
            isDesignMode: this.isDesignMode,
            viewerItemsManager: null
        };
        this._dashboardContext.beforeApplyViewerItemOptions.add((item, itemOptions, isCreation) => {
            itemOptions.useNeutralFilterMode = this.option('useNeutralFilterMode');
            itemOptions.encodeHtml = this.encodeHtml;
        });
        this._externalTemplates = ko.computed(() => {
            var result = [];
            this.customTemplates().forEach(template => {
                result.push(template);
            });
            this._extensions().forEach(extension => {
                if (extension['template']) {
                    if (typeof extension['template'] === 'string') {
                        result.push({
                            name: extension['template'],
                            data: extension
                        });
                    }
                    else {
                        result.push(extension['template']);
                    }
                }
            });
            return result;
        });
        this.notificationController = new NotificationController();
        this.isDesignMode.subscribe(isDesignMode => {
            this._validateWorkingMode();
        }, null, 'beforeChange');
        this.isDesignMode.subscribe(isDesignMode => {
            this._validateWorkingMode();
            this._initializeServiceClient(this.dashboardContainer());
        });
        this._allowSwitchToDesigner = this.option('workingMode') !== 'ViewerOnly';
        let extensionsToRegister = {};
        for (let name in defaultExtensions) {
            extensionsToRegister[name] = defaultExtensions[name];
        }
        if (this.allowSwitchToDesigner) {
            for (let name in designerExtensions) {
                extensionsToRegister[name] = designerExtensions[name];
            }
        }
        _setShowMessageFunc((msg, type, displayTime, debugInfo) => { });
        this._dockingLayoutAdapter = new DockingLayoutAdapter(this);
        this._dockingLayoutAdapter.start();
        this._raiseEvent('initializing', { component: this });
        this._viewerApi = new ViewerApi(this);
        this._viewerApi.start();
        this._registerDefaultExtensions(extensionsToRegister, this.option('extensions'));
        this._registerKeyProcessing();
        this._isLoading.subscribe(isLoading => this.notificationController.suspended(isLoading));
        this._isLoading(true);
        this._updateController.switchControlState('controlInitialized');
        setTimeout(() => {
            if (!this.dashboard()) {
                this._loadInitialDashboard()
                    .done(() => {
                    var initialDashboardState = this.option('initialDashboardState');
                    if (initialDashboardState) {
                        this.dashboard().stateString = initialDashboardState;
                    }
                })
                    .always(() => { this._isLoading(false); });
            }
            else {
                this._isLoading(false);
            }
        }, 1);
        _defineDashboardControlObsoleteMethods(this);
        if (this.option('renderImmediately')) {
            this.render();
        }
    }
    _updateLayout() {
        var binders = this._layoutInfoProviderCollection;
        var binder = binders.filter(b => b.condition())[0];
        if (binder && (!this._layoutInfo() || binder.name !== this._layoutInfo().template.name)) {
            this._layoutInfo({
                template: {
                    name: binder.name,
                    data: binder.getViewModel(),
                },
                layoutController: binder.getLayoutController(),
            });
        }
    }
    get _colorSchemeCss() {
        return getBaseColorScheme() === 'light' ? 'dx-color-scheme-light' : 'dx-color-scheme-dark';
    }
    repaint() {
        this._repaintRequest.fire();
    }
    render() {
        this._raiseEvent('beforeRender', { component: this });
        var openComment = document.createComment(" ko template: { name: 'dx-dashboard-control' } "), closingComment = document.createComment(' /ko ');
        this._element.appendChild(openComment);
        this._element.appendChild(closingComment);
        this._applyBindings(this._element);
    }
    maximizeDashboardItem(itemName) {
        if (this.dashboard()) {
            var dashboardItem = this.dashboard().findItem(itemName);
            if (dashboardItem) {
                var fullscreenItemProvider = this._getFullscreenItemProvider();
                if (fullscreenItemProvider) {
                    fullscreenItemProvider.maximizeItem(dashboardItem);
                }
            }
            else {
                throw Error("The item with the '" + itemName + "' name does not exist");
            }
        }
    }
    restoreDashboardItem() {
        var fullscreenItemProvider = this._getFullscreenItemProvider();
        fullscreenItemProvider && fullscreenItemProvider.restoreDownItem();
    }
    initializeDashboard(id, dashboardJson, initialState) {
        this._updateController.switchControlState('dashboardInitializing');
        if (this.option('dashboardId') !== id) {
            this.option('dashboardId', id);
        }
        this._unloadDashboardServices();
        var dashboard = new Dashboard(dashboardJson);
        if (initialState) {
            dashboard._state(initialState);
        }
        let e = {
            component: this,
            dashboard: dashboard,
            dashboardId: id,
            ready: createJQueryDeferred().resolve().promise()
        };
        this._raiseEvent('dashboardInitializing', e);
        if (dashboard.disposed)
            return;
        e.ready.done(() => {
            dashboard.items().forEach(item => item._useNeutralFilterMode(this.option('useNeutralFilterMode')));
            this._perDashboardDisposables.push(subscribeArrayChange(dashboard.items, {
                added: (newItem) => newItem._useNeutralFilterMode(this.option('useNeutralFilterMode'))
            }));
            var dashboardContainer = { id: id, dashboard: dashboard };
            this._initializeServiceClient(dashboardContainer);
            this._dataSourceBrowser = new DataSourceBrowser(dashboard.dataSources, this.isDesignMode, dashboard.parameters, this._serviceClient, this._isLoading);
            this._dashboardContext.viewerItemsManager = new ViewerItemAdaptersManager();
            this._updateHubPrivate = new DashboardUpdateHub(dashboard, this.option('dataRequestOptions'), {
                getItemData: (item) => this._serviceClient().getItemData(item, isModeAllowsToReduceData()),
                getBatchItemData: (items) => this._serviceClient().getBatchItemData(items, isModeAllowsToReduceData()),
                getMapShapeFile: (item) => this._serviceClient().getMapShapeFile(item.componentName()),
                clearDimensionValuesCache: () => this._dataSourceBrowser?.clearDimensionValuesCache()
            }, () => this._actualLayoutController() && this._actualLayoutController().visibleItemsProvider || null);
            this._updateHubPrivate.dashboardBeginUpdate = () => this._raiseEvent('dashboardBeginUpdate', {
                component: this,
                dashboardId: this.dashboardContainer().id
            });
            this._updateHubPrivate.dashboardEndUpdate = () => this._raiseEvent('dashboardEndUpdate', {
                component: this,
                dashboardId: this.dashboardContainer().id
            });
            this._updateHubPrivate.itemBeginUpdate = (itemName) => this._raiseEvent('itemBeginUpdate', {
                component: this,
                dashboardId: this.dashboardContainer().id,
                itemName: itemName
            });
            this._updateHubPrivate.itemEndUpdate = (itemName) => this._raiseEvent('itemEndUpdate', {
                component: this,
                dashboardId: this.dashboardContainer().id,
                itemName: itemName
            });
            this.dashboardContainer(dashboardContainer);
            this.notificationController.reset();
            this._updateController.switchControlState('dashboardInitialized');
            this._raiseEvent('dashboardInitialized', {
                component: this,
                dashboardId: id,
                dashboard: dashboard
            });
            if (dashboard.disposed)
                return;
            this._raiseEvent('dashboardStateChanged', {
                component: this,
                dashboard: dashboard,
                dashboardId: id,
                stateString: dashboard.stateString
            });
            if (dashboard.disposed)
                return;
            let isModeAllowsToReduceData = () => {
                var limitMode = this.option('limitVisibleDataMode');
                return limitMode === 'DesignerAndViewer' || (this.isDesignMode() && limitMode === 'Designer');
            };
            this._updateHubPrivate.initialize();
            this._perDashboardDisposables.push(this.dashboard()._state.subscribe((_) => {
                this._raiseEvent('dashboardStateChanged', {
                    component: this,
                    dashboard: this.dashboard(),
                    dashboardId: this.dashboardContainer().id,
                    stateString: dashboard.stateString
                });
            }));
        }).fail(() => {
            this.notificationController.reset();
        });
    }
    requestDashboardList() {
        var urls = this._endpointCollection.dashboardUrls;
        if (urls) {
            return this.remoteService.getFromServer(urls.GetDashboardsAction, null, { cache: false })
                .fail(() => {
                this.notificationController.showError(getLocalizationById('DashboardWebStringId.Notification.AttemptToLoadDashboards'));
            });
        }
        return createJQueryDeferred().reject().promise();
    }
    loadDashboard(dashboardId) {
        var def = createJQueryDeferred();
        if (this._dashboardLoadingDeferred) {
            this._dashboardLoadingDeferred.reject();
        }
        this._dashboardLoadingDeferred = def;
        if (this.option('dashboardId') === dashboardId) {
            this._loadDashboardCore(dashboardId);
        }
        else {
            this.option('dashboardId', dashboardId);
        }
        return def.promise();
    }
    _loadDashboardCore(dashboardId) {
        let isValidRequest = requestParamsValidator.isValid({ dashboardId });
        var urls = this._endpointCollection && this._endpointCollection.dashboardUrls || null;
        if (urls && isValidRequest) {
            const prevDashboardState = this._updateController.getControlState();
            this._updateController.switchControlState('dashboardLoading');
            this.notificationController.showState(getLocalizationById('DashboardWebStringId.Notification.DashboardLoading'));
            return this.remoteService.getFromServer(urls.DashboardAction + '/' + encodeURIComponent(dashboardId), null, { cache: false })
                .done((result) => {
                _addLicenseTrigger(result._licenseOptions);
                this._updateController.switchControlState('dashboardLoaded');
                if (this.option('dashboardId') === dashboardId) {
                    this.initializeDashboard(dashboardId, result.Dashboard, result.initialState);
                    this._dashboardLoadingDeferred && this._dashboardLoadingDeferred.resolve();
                }
            }).fail((val) => {
                this._updateController.switchControlState(prevDashboardState);
                this._dashboardLoadingDeferred && this._dashboardLoadingDeferred.reject();
                this.notificationController.showError(getLocalizationById('DashboardWebStringId.Notification.AttemptToLoadDashboard') + dashboardId, val);
            });
        }
        this._dashboardLoadingDeferred && this._dashboardLoadingDeferred.reject();
        this.notificationController.showError(getLocalizationById('DashboardWebStringId.Notification.AttemptToLoadDashboard') + String(dashboardId));
        return createJQueryDeferred().reject().promise();
    }
    unloadDashboard() {
        this.option('dashboardId', null);
    }
    _unloadDashboardCore() {
        this._unloadDashboardServices();
        this.dashboardContainer(null);
    }
    switchToViewer() {
        this.option('workingMode', 'Viewer');
    }
    _switchToViewer() {
        var actions = this.extensions
            .filter(extension => !!extension.designerToViewerAction)
            .map(extension => toCancelableSequenceAction(extension.designerToViewerAction))
            .sort((a, b) => a.orderNo - b.orderNo);
        let groups = groupByProperty(actions, 'orderNo');
        this._workingModeChangeDeferred.reject();
        this._workingModeChangeDeferred = createJQueryDeferred();
        return DashboardControl._recursiveAsyncEval(groups, new CancellationToken(this._workingModeChangeDeferred), { surfaceLeft: this._surfaceLeft(), surfaceTop: this._surfaceTop() })
            .then(options => {
            this.surfaceLeft(options.surfaceLeft);
            this.surfaceTop(options.surfaceTop);
            this.isDesignMode(false);
            this._workingModeChangeDeferred.resolve();
        }, () => this.switchToDesigner());
    }
    switchToDesigner() {
        this.option('workingMode', 'Designer');
    }
    _switchToDesigner() {
        this._validateWorkingMode();
        var actions = this.extensions
            .filter(extension => !!extension.viewerToDesignerAction)
            .map(extension => toCancelableSequenceAction(extension.viewerToDesignerAction))
            .sort((a, b) => a.orderNo - b.orderNo);
        let groups = groupByProperty(actions, 'orderNo');
        this._workingModeChangeDeferred.reject();
        this._workingModeChangeDeferred = createJQueryDeferred();
        return DashboardControl._recursiveAsyncEval(groups, new CancellationToken(this._workingModeChangeDeferred), { surfaceLeft: this._surfaceLeft(), surfaceTop: this._surfaceTop() })
            .then((options) => {
            this.surfaceLeft(options.surfaceLeft);
            this.surfaceTop(options.surfaceTop);
            this.isDesignMode(true);
            this._workingModeChangeDeferred.resolve();
        }, () => this.switchToViewer());
    }
    reloadData() {
        if (this.dashboard()) {
            this._serviceClient().markDataSourcesForReload().done((result) => {
                this.refresh();
            });
        }
    }
    refresh(args) {
        if (this.dashboard()) {
            var itemNames = [];
            if (!args) {
                itemNames = this.dashboard()._dataDashboardItems().map(item => item.componentName());
            }
            else if (typeof args === 'string') {
                itemNames = [args];
            }
            else if (Array.isArray(itemNames)) {
                itemNames = args;
            }
            else {
                throw Error('Invalid arguments');
            }
            this._updateHubPrivate.refreshItems(itemNames);
        }
    }
    getDashboardState() {
        var dashboard = this.dashboard();
        if (dashboard)
            return dashboard.stateString;
        return null;
    }
    setDashboardState(dashboardState) {
        var dashboard = this.dashboard();
        if (dashboard) {
            if (typeof dashboardState === 'string') {
                dashboard.stateString = dashboardState;
            }
            else {
                dashboard._state(dashboardState);
            }
        }
    }
    getDashboardId() {
        var dashboardContainer = this.dashboardContainer();
        if (dashboardContainer)
            return dashboardContainer.id;
        return null;
    }
    registerIcon(icon, id) {
        ResourceManager.registerIcon(icon, id);
    }
    registerExtension(...extensions) {
        extensions.forEach(extension => {
            if (this._canAddExtension(extension))
                this._registerExtensionCore(extension);
        });
    }
    _registerExtensionCore(extension) {
        const camelCaseExtensionName = this._getCamelCaseExtensionName(extension.name);
        const optionsManager = extension._optionsManager;
        if (optionsManager) {
            if (typeof this._controlOptions.extensions !== 'object') {
                this._controlOptions.extensions = {};
            }
            this._controlOptions.extensions[camelCaseExtensionName] = optionsManager.getInitialOptions();
            this._defaultControlOptions.extensions[camelCaseExtensionName] = optionsManager.getDefaultOptions();
        }
        else {
            this._controlOptions.extensions = this._controlOptions.extensions || {};
            this._controlOptions.extensions[camelCaseExtensionName] = {};
        }
        this._addExtension(extension);
        this._startExtension(extension);
    }
    findExtension(extensionName) {
        const camelName = this._getCamelCaseExtensionName(extensionName);
        return this._extensions().find(ext => ext.name === camelName);
    }
    unregisterExtension(...extensionNames) {
        extensionNames.forEach(name => {
            const camelName = this._getCamelCaseExtensionName(name);
            this.option(`extensions.${camelName}`, false);
            this._defaultControlOptions.extensions[camelName] = false;
        });
    }
    _unregisterExtensionCore(...extensionNames) {
        extensionNames.forEach(extensionName => {
            var extension = this.findExtension(extensionName);
            if (!!extension) {
                var customItemExtension = extension;
                if (customItemExtension.createViewerItem) {
                    this._dashboardContext.viewerItemCreator[extension.name] = undefined;
                }
                if (customItemExtension.metaData) {
                    delete itemTypesMap[extension.name];
                }
                extension.stop && extension.stop();
                this._extensions.remove(extension);
            }
        });
    }
    subscribeExtensionsChanged(handlers) {
        return subscribeArrayChange(this._extensions, handlers);
    }
    dispose() {
        this._unloadDashboardCore();
        this._viewerApi.stop();
        this._dockingLayoutAdapter.dispose();
        $.fn.constructor(document).off('.dx-dshd');
        ko.cleanNode(this._element);
        this._element.innerHTML = '';
    }
    _renderDashboardItem(element, dashboardItemName, isStandalone = true) {
        var openComment = document.createComment(" ko template: { name: 'dx-dashboard-standalone-item' } "), closingComment = document.createComment(' /ko ');
        element.appendChild(openComment);
        element.appendChild(closingComment);
        var dashboardItem = ko.computed(() => this.dashboard() && this.dashboard().items().filter(i => i.componentName.peek() === dashboardItemName)[0]);
        this._applyBindings(element, {
            dashboardItem: dashboardItem,
            context: this._dashboardContext,
            sizeController: new SingleItemSizeController(element, this._repaintRequest),
            isStandalone: isStandalone
        });
    }
    _getFullscreenItemProvider() {
        return this._actualLayoutController() && this._actualLayoutController().fullscreenItemProvider;
    }
    _registerKeyProcessing() {
        _staticInitialize();
        var _ignoreKeyProcessing = () => {
            if (!this.isDesignMode() || !this._actualLayoutController()) {
                return true;
            }
            var activeElement = $.fn.constructor(document.activeElement);
            if (activeElement.is('textarea') || activeElement.is(':input') && (['password', 'text', 'number'].indexOf(activeElement.attr('type')) != -1)) {
                return true;
            }
            return false;
        };
        $.fn.constructor(document).on('keyup.dx-dshd', (e) => {
            if (_ignoreKeyProcessing()) {
                return;
            }
            if (this._dockingLayoutAdapter.processKeyEvent && this._dockingLayoutAdapter.processKeyEvent('keyup', e)) {
                return false;
            }
            var i;
            for (i = 0; i < this._extensions().length; i++) {
                var extension = this._extensions()[i];
                if (extension.processKeyEvent && extension.processKeyEvent('keyup', e)) {
                    return false;
                }
            }
        });
        $.fn.constructor(document).on('keydown.dx-dshd', (e) => {
            if (_ignoreKeyProcessing()) {
                return;
            }
            var i;
            for (i = 0; i < this._extensions().length; i++) {
                var extension = this._extensions()[i];
                if (extension.processKeyEvent && extension.processKeyEvent('keydown', e)) {
                    return false;
                }
            }
        });
    }
    _registerDefaultExtensions(extensions, extensionOptions = {}) {
        if (extensionOptions !== false) {
            for (let extensionName in extensions) {
                const camelName = this._getCamelCaseExtensionName(extensionName);
                let options = extensionOptions[camelName];
                if (options !== false) {
                    if (!!options && typeof options !== 'object') {
                        throw Error('Extension options ');
                    }
                    let extension = extensions[extensionName](this, options);
                    this._canAddExtension(extension) && this._addExtension(extension);
                    var optionsManager = extension._optionsManager;
                    this._controlOptions.extensions[camelName] = deepExtend({}, optionsManager && optionsManager.getInitialOptions(), options);
                    this._defaultControlOptions.extensions[camelName] = optionsManager && optionsManager.getDefaultOptions();
                }
            }
            if (typeof extensionOptions === 'object') {
                for (let extensionName in extensionOptions) {
                    const camelName = this._getCamelCaseExtensionName(extensionName);
                    let options = extensionOptions[camelName];
                    if (typeof options === 'function') {
                        let extensionCreator = options;
                        let extension = extensionCreator(this);
                        this._canAddExtension(extension) && this._addExtension(extension);
                        var optionsManager = extension._optionsManager;
                        this._controlOptions.extensions[camelName] = optionsManager && optionsManager.getInitialOptions();
                        this._defaultControlOptions.extensions[camelName] = optionsManager && optionsManager.getDefaultOptions();
                    }
                }
            }
        }
        this.extensions.forEach(extension => this._startExtension(extension));
    }
    _unrollEndpointCollection(options) {
        if (options['endpointCollection']) {
            return options['endpointCollection'];
        }
        else if (options.endpoint) {
            return {
                dashboardUrls: {
                    DashboardAction: options.endpoint + '/dashboards',
                    GetDashboardsAction: options.endpoint + '/dashboards'
                },
                dataSourceUrls: {
                    GetDataSourcesAction: options.endpoint + '/dataSources'
                },
                dataSourceWizardUrls: {
                    DataSourceWizardAction: options.endpoint + '/data/DataSourceWizardAction',
                    GetConnectionStringsAction: options.endpoint + '/data/GetConnectionStringsAction'
                },
                dataServiceUrls: {
                    ConvertItemAction: options.endpoint + '/data/ConvertItemAction',
                    DashboardItemGetAction: options.endpoint + '/data/DashboardItemGetAction',
                    DimensionFilterItemsAction: options.endpoint + '/data/DimensionFilterItemsAction',
                    DimensionFilterStringAction: options.endpoint + '/data/DimensionFilterStringAction',
                    DimensionUniqueValuesAction: options.endpoint + '/data/DimensionUniqueValuesAction',
                    FieldListAction: options.endpoint + '/data/FieldListAction',
                    GetColoringSchemeAction: options.endpoint + '/data/GetColoringSchemeAction',
                    GetDashboardPaletteAction: options.endpoint + '/data/GetDashboardPaletteAction',
                    GetMapShapeFileAction: options.endpoint + '/data/GetMapShapeFileAction',
                    GetUnderlyingDataAction: options.endpoint + '/data/GetUnderlyingDataAction',
                    MarkDataSourcesForReloadAction: options.endpoint + '/data/MarkDataSourcesForReloadAction',
                    ParameterValuesAction: options.endpoint + '/data/ParameterValuesAction',
                    PerformExportAction: options.endpoint + '/data/PerformExportAction',
                    DashboardItemBatchGetAction: options.endpoint + '/data/DashboardItemBatchGetAction',
                    GetAvailableFontFamiliesAction: options.endpoint + '/data/GetAvailableFontFamiliesAction'
                }
            };
        }
        else {
            return {};
        }
    }
    _loadInitialDashboard() {
        if (!!this.option('dashboardId')) {
            return this._loadDashboardCore(this.option('dashboardId'));
        }
        else if (!!this.option('initialDashboardId')) {
            return this.loadDashboard(this.option('initialDashboardId'));
        }
        else if (!!this.option('loadDefaultDashboard')) {
            var res = createJQueryDeferred();
            this.requestDashboardList()
                .done(items => {
                if (items[0]) {
                    this.loadDashboard(items[0].id)
                        .done(() => res.resolve())
                        .fail(() => res.reject());
                }
                else {
                    res.reject();
                }
            })
                .fail(() => {
                res.reject();
            });
            return res.promise();
        }
        else {
            return createJQueryDeferred().resolve().promise();
        }
    }
    _initializeServiceClient(dashboardContainer) {
        var serviceClient = this.isDesignMode() ?
            !this._validateWorkingMode() && new DesignerDataServiceClient(dashboardContainer, this.notificationController, this._endpointCollection.dataServiceUrls, this.remoteService) || null :
            new ViewerDataServiceClient(dashboardContainer, this.notificationController, this._endpointCollection.dataServiceUrls, this.remoteService);
        this._serviceClient(serviceClient);
    }
    _validateWorkingMode() {
        if (!this.allowSwitchToDesigner) {
            throw Error('Cannot switch to Designer from the ViewerOnly mode.');
        }
        return false;
    }
    _canAddExtension(extension) {
        if (!!this.findExtension(extension.name)) {
            console.error("The extension with the '" + extension.name + "' name is already registered.");
            return false;
        }
        return true;
    }
    _addExtension(extension) {
        this._extensions.push(extension);
        var customItemExtension = extension;
        if (customItemExtension.createViewerItem) {
            this._dashboardContext.viewerItemCreator[extension.name] = customItemExtension.createViewerItem;
        }
        if (customItemExtension.metaData) {
            itemTypesMap[extension.name] = deepExtend({
                type: CustomItem,
                customItemType: extension.name,
            }, customItemExtension.metaData);
        }
    }
    _startExtension(extension) {
        extension.start && extension.start();
    }
    _unloadDashboardServices() {
        var dashboard = this.dashboard();
        this._perDashboardDisposables.forEach(d => d.dispose());
        this._perDashboardDisposables.splice(0, this._perDashboardDisposables.length);
        if (dashboard) {
            dashboard.dispose();
            this._dashboardContext.viewerItemsManager.dispose();
            if (this._updateHubPrivate) {
                this._updateHubPrivate.dispose();
                this._updateHubPrivate = null;
            }
            if (this._dataSourceBrowser) {
                this._dataSourceBrowser.dispose();
                this._dataSourceBrowser = null;
            }
        }
    }
    _applyBindings(element, additionalProperties) {
        var viewModel = {
            getWidgetContainer: this.getWidgetContainer,
            surfaceLeft: this._surfaceLeft,
            surfaceTop: this._surfaceTop,
            colorSchemeCss: this._colorSchemeCss,
            classList: () => this._classList,
            isDashboardLoaded: ko.computed(() => !!this.dashboard()),
            layoutTemplate: safeComputed({ info: this._layoutInfo }, ({ info }) => info.template),
            externalTemplates: this._externalTemplates,
            emptyControlTemplates: this._emptyControlTemplates,
            notificationController: this.notificationController,
            isLoading: this._isLoading,
            getLocalizationById: getLocalizationById,
            extend: extend,
            $unwrap: $unwrap,
            $: $,
            ko: ko,
            ...additionalProperties
        };
        appendStaticContextToRootViewModel(viewModel);
        ko.applyBindings(viewModel, element);
    }
    _remoteServiceFactory() {
        return this._createRemoteService({
            ajaxRemoteService: this.option('ajaxRemoteService'),
            fetchRemoteService: this.option('fetchRemoteService')
        });
    }
    _createRemoteService(option) {
        if (type.isDefined(option.ajaxRemoteService) && type.isDefined(option.fetchRemoteService)) {
            throw new Error('You cannot register more than one type of remote service. Use either ajaxRemoteService or fetchRemoteService.');
        }
        if (type.isDefined(option.ajaxRemoteService)) {
            return new AjaxRemoteService(option.ajaxRemoteService);
        }
        return new FetchRemoteService(option.fetchRemoteService);
    }
}
ko.bindingHandlers['element-height-change-provider'] = {
    init: (el, valueAccessor, allbindings, viewmodel, bindingcontext) => {
        var data = ko.unwrap(valueAccessor());
        var recalculate = () => {
            setTimeout(() => {
                data.height(getHeight(el));
            }, 0);
        };
        data.model.subscribe(function () {
            recalculate();
        });
        recalculate();
    }
};
ko.bindingHandlers['dxAttach2Model'] = {
    init: function (element, valueAccessor, allBindings, viewModel, bindingContext) {
        var attachmentName = ko.unwrap(valueAccessor());
        if (['getLookupDefaultValuesSelectBoxOptions', 'getLookupDefaultValueSelectBoxOptions', 'dataSourceBrowser'].indexOf(attachmentName) === -1) {
            throw 'non supported';
        }
        var findAttachment = (name) => {
            var context = bindingContext.$parents.filter((item) => { return item[name] !== undefined; })[0];
            if (context) {
                return context[name];
            }
            return null;
        }, attachment = findAttachment(attachmentName);
        if (attachment) {
            viewModel[attachmentName] = attachment;
        }
    }
};
function _staticInitialize() {
    dxPopup.defaultOptions({
        options: {
            onInitialized: (e) => {
                const popup = e.component;
                popup.registerKeyHandler('escape', function (e) {
                    e.originalEvent.stopPropagation();
                    popup.hide();
                });
            }
        }
    });
}
function _defineDashboardControlObsoleteMethods(control) {
    defineObsoleteProperty({
        target: control,
        memberName: 'toolbox',
        oldMemberDisplayName: 'DashboardControl.toolbox',
        newMemberDisplayName: 'ToolboxExtension.toolboxGroups',
        action: () => {
            var toolboxExtension = control.findExtension('toolbox');
            if (toolboxExtension) {
                return toolboxExtension.toolboxGroups;
            }
            else {
                console.error('The ToolboxExtension could not be found.');
            }
        }
    });
    defineObsoleteProperty({
        target: control,
        memberName: 'toolbarItems',
        oldMemberDisplayName: 'DashboardControl.toolbarItems',
        newMemberDisplayName: 'ToolboxExtension.toolbarGroups',
        action: () => {
            var toolboxExtension = control.findExtension('toolbox');
            if (toolboxExtension) {
                return toolboxExtension.toolbarGroups;
            }
            else {
                console.error('The ToolboxExtension could not be found.');
            }
        }
    });
    defineObsoleteProperty({
        target: control,
        memberName: 'menuItems',
        oldMemberDisplayName: 'DashboardControl.menuItems',
        newMemberDisplayName: 'ToolboxExtension.menuItems',
        action: () => {
            var toolboxExtension = control.findExtension('toolbox');
            if (toolboxExtension) {
                return toolboxExtension.menuItems;
            }
            else {
                console.error('The ToolboxExtension could not be found.');
            }
        }
    });
    defineObsoleteProperty({
        target: control,
        memberName: 'menuVisible',
        oldMemberDisplayName: 'DashboardControl.menuVisible',
        newMemberDisplayName: 'ToolboxExtension.menuVisible',
        action: () => {
            var toolboxExtension = control.findExtension('toolbox');
            if (toolboxExtension) {
                return toolboxExtension.menuVisible;
            }
            else {
                console.error('The toolboxExtension could not be found.');
            }
        }
    });
    defineObsoleteProperty({
        target: control,
        memberName: 'selectMenuItem',
        oldMemberDisplayName: 'DashboardControl.selectMenuItem',
        newMemberDisplayName: 'ToolboxExtension.selectMenuItem',
        action: () => {
            var toolboxExtension = control.findExtension('toolbox');
            if (toolboxExtension) {
                return toolboxExtension.selectMenuItem;
            }
            else {
                console.error('The toolboxExtension could not be found.');
            }
        }
    });
    defineObsoleteProperty({
        target: control,
        memberName: 'undoEngine',
        oldMemberDisplayName: 'DashboardControl.undoEngine',
        newMemberDisplayName: 'UndoRedoExtension.undoEngine',
        action: () => {
            var undoEngineExtension = control.findExtension('undoRedo');
            if (undoEngineExtension) {
                return undoEngineExtension.undoEngine;
            }
            else {
                console.error('The UndoRedoExtension could not be found.');
            }
        }
    });
    defineObsoleteProperty({
        target: control,
        memberName: 'colorSchemeCss',
        warmMessage: 'The DashboardControl.colorSchemeCss is obsolete. Use the DevExpress.ui.themes.current method instead.',
        action: () => control._colorSchemeCss
    });
    defineObsoleteMethod({
        target: control,
        memberName: 'saveDashboard',
        oldMemberDisplayName: 'DashboardControl.saveDashboard',
        newMemberDisplayName: 'SaveDashboardExtension.saveDashboard',
        action: () => {
            var saveDashboardExtension = control.findExtension('saveDashboard');
            if (saveDashboardExtension) {
                return saveDashboardExtension.saveDashboard();
            }
            else {
                console.error('The SaveDashboardExtension could not be found.');
            }
        }
    });
    defineObsoleteMethod({
        target: control,
        memberName: 'ensureDashboardSaved',
        oldMemberDisplayName: 'DashboardControl.ensureDashboardSaved',
        newMemberDisplayName: 'SaveDashboardExtension.ensureDashboardSaved',
        action: (action) => {
            var saveDashboardExtension = control.findExtension('saveDashboard');
            if (saveDashboardExtension) {
                return saveDashboardExtension.ensureDashboardSaved(action);
            }
            else {
                console.error('The SaveDashboardExtension could not be found.');
            }
        }
    });
    defineObsoleteMethod({
        target: control,
        memberName: 'clearDesigner',
        oldMemberDisplayName: 'DashboardControl.clearDesigner',
        newMemberDisplayName: 'DashboardControl.unloadDashboard',
        action: () => control.unloadDashboard()
    });
    defineObsoleteMethod({
        target: control,
        memberName: 'close',
        oldMemberDisplayName: 'DashboardControl.close',
        newMemberDisplayName: 'DashboardControl.unloadDashboard',
        action: () => control.unloadDashboard()
    });
}
addToBindingsCache('template: { name: \'dx-dashboard-control\' }', function () {
    return {
        'template': function () {
            return { name: 'dx-dashboard-control' };
        }
    };
});
addToBindingsCache('dxControlsDescendantBindings: true', function () {
    return {
        'dxControlsDescendantBindings': function () {
            return true;
        }
    };
});
