﻿/**
* DevExpress Dashboard (notificator.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { decodeHtml } from '../../data/_utils';
import { AriaLiveRegion } from './_aria-live-region';
import { NotificationControllerViewModel } from './_notificator-view-model';
export class NotificationController {
    constructor() {
        this._viewModel = new NotificationControllerViewModel();
    }
    static _getErrorTextFromResponse(request) {
        return (request && request.responseJSON && request.responseJSON.Message) || '';
    }
    static _getDetailedErrorMessage(errorInfo) {
        var errorDetail = '';
        if (typeof errorInfo == 'string') {
            errorDetail = errorInfo;
        }
        else if (errorInfo && errorInfo['responseJSON']) {
            errorDetail = NotificationController._getErrorTextFromResponse(errorInfo);
        }
        return decodeHtml(errorDetail);
    }
    suspended(isSuspended) {
        this._viewModel.suspended(isSuspended);
    }
    showState(message) {
        this._viewModel.updateNotification('info', message);
        AriaLiveRegion.updateLiveRegion(message, false);
    }
    showSuccess(message) {
        this._viewModel.updateNotification('success', message);
        AriaLiveRegion.updateLiveRegion(message, false);
    }
    showError(title, errorInfo) {
        let errorDetail = NotificationController._getDetailedErrorMessage(errorInfo);
        this._viewModel.updateNotification('error', title, errorDetail);
        AriaLiveRegion.updateLiveRegion(title, true);
        console.warn(title + (errorDetail ? ' - ' + errorDetail : ''));
    }
    reset() {
        this._viewModel.reset();
    }
}
