﻿/**
* DevExpress Dashboard (resource-manager.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { SvgTemplatesEngine } from '@devexpress/analytics-core/analytics-widgets-internal';
import { setLocalization } from '../data/localization/_localization-initializer';
import { defineObsoleteMethod } from '../model/internal/_obsolete-helper';
export class ResourceManager {
    static setLocalizationMessages(localizationMessages) {
        setLocalization(localizationMessages);
    }
    static embedBundledResources() {
    }
    static removeEmbeddedResources() {
    }
    static registerIcon(icon, id) {
        const html = typeof icon !== 'string' ? icon.outerHTML : icon;
        let imageId = id;
        if (!imageId) {
            const tmpElt = document.createElement('div');
            tmpElt.innerHTML = html;
            const svgElts = Array.from(tmpElt.querySelectorAll('svg'));
            if (svgElts.length !== 1) {
                console.error(`The icon can not consist of multiple SVG elements. Make sure that the icon is a single SVG element. The SVG element is '${html}'.`);
                return;
            }
            const svgId = svgElts[0].getAttribute('id');
            if (!svgId) {
                console.error(`The icon cannot be registered. Add the id attribute to the icon’s definition or pass the icon's id as the second parameter to the registerIcon method. The SVG element is '${html}'.`);
                return;
            }
            imageId = svgId;
        }
        SvgTemplatesEngine.addTemplate(imageId, html);
    }
}
defineObsoleteMethod({
    target: ResourceManager,
    memberName: 'removeEmbededResources',
    oldMemberDisplayName: 'ResourceManager.removeEmbededResources',
    newMemberDisplayName: 'ResourceManager.removeEmbeddedResources',
    action: () => { }
});
defineObsoleteMethod({
    target: ResourceManager,
    memberName: 'embedBundledResources',
    oldMemberDisplayName: 'ResourceManager.embedBundledResources',
    warmMessage: 'The ResourceManager.embedBundledResources() method is obsolete. Remove the method call from your application. For more information, refer to the following breaking change document: https://supportcenter.devexpress.com/ticket/details/T1249022.',
    action: () => { }
});
defineObsoleteMethod({
    target: ResourceManager,
    memberName: 'removeEmbeddedResources',
    oldMemberDisplayName: 'ResourceManager.removeEmbeddedResources',
    warmMessage: 'The ResourceManager.removeEmbeddedResources() method is obsolete. Remove the method call from your application. For more information, refer to the following breaking change document: https://supportcenter.devexpress.com/ticket/details/T1249022.',
    action: () => { }
});
