﻿/**
* DevExpress Dashboard (_factory.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { types } from '../viewer-parts/_viewer-item-types';
import { bubbleMapDataController } from './data-controllers/_bubble-map-data-controller';
import { cardDataController } from './data-controllers/_card-data-controller';
import { chartDataController } from './data-controllers/_chart-data-controller';
import { choroplethMapDataController } from './data-controllers/_choropleth-map-data-controller';
import { listViewDataController, treeViewDataController } from './data-controllers/_filter-element-data-controller';
import { gaugeDataController } from './data-controllers/_gauge-data-controller';
import { geoPointMapDataController } from './data-controllers/_geo-point-map-data-controller';
import { gridDataController } from './data-controllers/_grid-data-controller';
import { imageDataController } from './data-controllers/_image-data-controller';
import { pieDataController } from './data-controllers/_pie-data-controller';
import { pieMapDataController } from './data-controllers/_pie-map-data-controller';
import { pivotDataController } from './data-controllers/_pivot-data-controller';
import { rangeFilterDataController } from './data-controllers/_range-filter-data-controller';
import { scatterChartDataController } from './data-controllers/_scatter-chart-data-controller';
import { textItemDataController } from './data-controllers/_text-item-data-controller';
import { treemapDataController } from './data-controllers/_treemap-data-controller';
export class DataControllerFactory {
    createDataController(type, options) {
        switch (type) {
            case types.rangeFilter:
                return new rangeFilterDataController(options);
            case types.chart:
                return new chartDataController(options);
            case types.scatter:
                return new scatterChartDataController(options);
            case types.pie:
                return new pieDataController(options);
            case types.pivot:
                return new pivotDataController(options);
            case types.choroplethMap:
                return new choroplethMapDataController(options);
            case types.grid:
                return new gridDataController(options);
            case types.card:
                return new cardDataController(options);
            case types.gauge:
                return new gaugeDataController(options);
            case types.geoPointMap:
                return new geoPointMapDataController(options);
            case types.bubbleMap:
                return new bubbleMapDataController(options);
            case types.pieMap:
                return new pieMapDataController(options);
            case types.comboBox:
            case types.listBox:
                return new listViewDataController(options);
            case types.treeView:
                return new treeViewDataController(options);
            case types.boundImage:
                return new imageDataController(options);
            case types.text:
                return new textItemDataController(options);
            case types.treemap:
                return new treemapDataController(options);
            default:
                return undefined;
        }
    }
}
export let defaultDataControllerFactory = new DataControllerFactory();
