﻿/**
* DevExpress Dashboard (_gauge-data-controller.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { gaugeRangeCalculator } from '../_gauge-range-calculator';
import { KpiValueMode } from '../_utils';
import { kpiDataController } from './_kpi-data-controller';
export class gaugeDataController extends kpiDataController {
    constructor(options) {
        super(options);
        this._gaugeRanges = {};
    }
    _iterateKpiItems(delegate) {
        var that = this;
        if (that.viewModel) {
            that.viewModel.Gauges.forEach(gauge => {
                delegate(gauge);
            });
        }
    }
    _getGaugeRange(element) {
        var elementId = element.ID, range = this._gaugeRanges[elementId], calculator;
        if (!range) {
            calculator = new gaugeRangeCalculator({
                values: this._getGaugeValues(element),
                gaugeModel: {
                    Type: this._gaugeViewType,
                    MinValue: element.MinValue,
                    MaxValue: element.MaxValue
                }
            });
            range = calculator.getGaugeRange();
            this._gaugeRanges[elementId] = range;
        }
        return range;
    }
    _getGaugeValues(element) {
        var multiData = this.multiData, gaugeValues = [], axisPoints = this._axisPoints || [null], getMeasureValue = function (axisPoint) {
            var getMeasure = axisPoint ? multiData.getMeasureValueByAxisPoints : multiData.getMeasureValue;
            gaugeValues.push(getMeasure.call(multiData, element.ID, [axisPoint]).getValue());
        }, getDeltaValue = function (axisPoint) {
            var getDelta = axisPoint ? multiData.getDeltaValueByAxisPoints : multiData.getDeltaValue, deltaValue = getDelta.call(multiData, element.ID, [axisPoint]), actualValue = deltaValue.getActualValue(), targetValue = deltaValue.getTargetValue();
            gaugeValues.push(actualValue.getValue());
            gaugeValues.push(targetValue.getValue());
        }, getter = element.DataItemType === KpiValueMode.Measure ? getMeasureValue : getDeltaValue;
        axisPoints.forEach(axisPoint => {
            getter(axisPoint);
        });
        return gaugeValues;
    }
    _initialize() {
        super._initialize();
        this._gaugeViewType = this.viewModel ? this.viewModel.ViewType : undefined;
    }
}
