﻿/**
* DevExpress Dashboard (_range-filter-data-controller.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { chartHelper } from '../_chart-helper';
import { convertToFormat } from '../_formatter';
import { localizationId } from '../_localization-ids';
import { localizer } from '../_localizer';
import { chartDataControllerProto } from './_chart-data-controller-proto';
export class rangeFilterDataController extends chartDataControllerProto {
    constructor(options) {
        super(options);
    }
    getArgument(argumentAxisPoint) {
        if (argumentAxisPoint.getParent() != null) {
            return argumentAxisPoint.getValue();
        }
        else {
            return localizer.getString(localizationId.ChartTotalValue);
        }
    }
    _iterateSeriesTemplates(proc) {
        this.viewModel.SeriesTemplates.forEach(seriesTemplate => {
            proc(undefined, seriesTemplate);
        });
    }
    _isSelectionTagsRequired() {
        return false;
    }
    _getLastSeriesType(colorMeasureId) {
        var seriesTemplates = this.viewModel.SeriesTemplates;
        for (var i = seriesTemplates.length - 1; i >= 0; i--) {
            if (colorMeasureId === seriesTemplates[i].ColorMeasureID) {
                return chartHelper.convertSeriesType(seriesTemplates[i].SeriesType);
            }
        }
    }
    _createArgumentFormat() {
        const argument = this.viewModel ? this.viewModel.Argument : undefined;
        let argumentAxisLabelFormat = undefined;
        if (argument && !this.isDiscreteArgument()) {
            argumentAxisLabelFormat = convertToFormat(argument.AxisXLabelFormat);
            if (this._isNumericDataType(argument.Type)) {
                if (!argumentAxisLabelFormat || !argument.AxisXLabelFormat.NumericFormat) {
                    argumentAxisLabelFormat = this._getArgumentAutoFormat();
                }
                else if (argument.AxisXLabelFormat.NumericFormat.Unit == 'Auto') {
                    argumentAxisLabelFormat = { ...this._getArgumentAutoFormat(), format: argumentAxisLabelFormat.format };
                }
                argumentAxisLabelFormat.dateType = null;
            }
            if (argumentAxisLabelFormat) {
                argumentAxisLabelFormat.showTrailingZeros = false;
            }
        }
        else {
            argumentAxisLabelFormat = super._createArgumentFormat();
        }
        return argumentAxisLabelFormat;
    }
    showPointLabels(pointLabelInfo) {
        return false;
    }
    getArgumentBindingValue(argumentPoint, pointIndex) {
        return argumentPoint.getValue();
    }
}
