﻿/**
* DevExpress Dashboard (_button-group.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { getTemplate } from '@devexpress/analytics-core/analytics-utils';
import * as ko from 'knockout';
import { getLocalizationById } from '../../data/localization/_default';
export const _registerDxDashboardButtonGroup = () => {
    const componentName = 'dx-dashboard-button-group';
    const templateName = 'dx-dashboard-button-group-template';
    if (ko.components.isRegistered(componentName))
        return;
    ko.components.register(componentName, {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var options = {
                    keyExpr: 'value',
                    width: '100%',
                    items: ko.unwrap(params.values).map(function (val) { return { value: val.value, text: getLocalizationById(val.displayValue) }; }),
                    onItemClick: function (e) { params.value(e.itemData.value); },
                    disabled: params.disabled,
                    selectedItemKeys: [ko.unwrap(params.value.peek())]
                };
                return options;
            }
        },
        template: getTemplate(templateName)
    });
};
export const _registerDxDashboardCheckedButton = () => {
    const componentName = 'dx-dashboard-checked-button';
    const templateName = 'dx-dashboard-button-group-template';
    if (ko.components.isRegistered(componentName))
        return;
    ko.components.register(componentName, {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                let key = 'key';
                var options = {
                    elementAttr: { class: params.icon ? 'dx-dashboard-button-with-icon' : undefined },
                    keyExpr: 'key',
                    width: '100%',
                    items: ko.computed(() => {
                        var template = ko.unwrap(params.template);
                        var icon = ko.unwrap(params.icon);
                        if (!template && icon) {
                            template = `<!-- ko template: '${icon}' --><!-- /ko -->`;
                        }
                        return [{
                                key: key,
                                disabled: ko.unwrap(params.disabled),
                                hint: ko.unwrap(params.hint),
                                text: ko.unwrap(params.text),
                                template: template
                            }];
                    }),
                    onItemClick: params.click,
                    disabled: params.disabled,
                    selectedItemKeys: ko.computed(() => ko.unwrap(params.isSelected) ? [key] : []),
                };
                if (params.isSelected === false) {
                    options.onSelectionChanged = (e) => {
                        e.component.option('selectedItemKeys', []);
                    };
                }
                return options;
            }
        },
        template: getTemplate(templateName)
    });
};
