﻿/**
* DevExpress Dashboard (_expression-editor-display-name-provider.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { createJQueryDeferred } from '../../data/_jquery-helpers';
import { EFDataSource } from '../../model/data-sources/ef-data-source';
import { ObjectDataSource } from '../../model/data-sources/object-data-source';
import { XpoDataSource } from '../../model/data-sources/xpo-data-source';
export class ExpressionEditorDisplayNameProvider {
    static create(dataSourceProvider, dataFieldsProvider, dataSourceName, dataMember) {
        var dataSource = dataSourceProvider.findDataSource(dataSourceName);
        if (dataSource instanceof ObjectDataSource || dataSource instanceof XpoDataSource || dataSource instanceof EFDataSource) {
            return new ExpressionEditorDisplayNameProvider(dataFieldsProvider, dataSourceName, dataMember);
        }
        return null;
    }
    constructor(_dataFieldsProvider, _dataSourceName, _dataMember) {
        this._dataFieldsProvider = _dataFieldsProvider;
        this._dataSourceName = _dataSourceName;
        this._dataMember = _dataMember;
    }
    getDisplayNameByPath(dataSourcePath, fieldPath) {
        let parts = fieldPath.split('.');
        let result = null;
        let curPath = null;
        for (let i = 0; i < parts.length; i++) {
            curPath = curPath ? curPath + '.' + parts[i] : parts[i];
            let closedCurPath = curPath;
            let addCurDisplayPart = (curDisplayPath) => this._dataFieldsProvider.findDataField(this._dataSourceName, this._dataMember, closedCurPath)
                .then(dataField => {
                let def = createJQueryDeferred();
                if (!dataField)
                    def.reject(fieldPath);
                else {
                    let displayName = dataField.displayName() || dataField.dataMember();
                    def.resolve(curDisplayPath ? curDisplayPath + '.' + displayName : displayName);
                }
                return def.promise();
            }, () => fieldPath);
            result = result ? result.then(addCurDisplayPart) : addCurDisplayPart('');
        }
        let removeRejection = (result) => createJQueryDeferred().resolve(result).promise();
        return result.then(removeRejection, removeRejection);
    }
    getRealName(dataSourcePath, fieldDisplayPath) {
        return this._getRealNameRecursive('', fieldDisplayPath.split('.'))
            .then(dataMember => createJQueryDeferred().resolve(dataMember ? dataMember : fieldDisplayPath).promise());
    }
    _getRealNameRecursive(curFieldPath, displayNameParts) {
        return this._dataFieldsProvider.getDataFieldsArray(this._dataSourceName, this._dataMember, curFieldPath, _ => true).then(dataFieldList => {
            let i = 0;
            let dataField;
            for (; i <= displayNameParts.length; i++) {
                const displayName = displayNameParts.slice(0, i).join('.');
                dataField = dataFieldList.filter(dataField => dataField.displayName() === displayName)[0];
                if (dataField)
                    break;
            }
            if (!dataField)
                return;
            if (i === displayNameParts.length)
                return dataField.dataMember();
            return this._getRealNameRecursive(dataField.dataMember(), displayNameParts.slice(i));
        });
    }
}
