﻿/**
* DevExpress Dashboard (create-dashboard.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { ModelSerializer } from '@devexpress/analytics-core/analytics-utils';
import * as ko from 'knockout';
import { designerExtensions, extensionNameMap } from '../../common/control-options';
import { getLocalizationById } from '../../data/localization/_default';
import { Dashboard } from '../../model/dashboard';
import { DashboardMenuItem } from '../toolbox-extension/toolbox-items';
const name = 'createDashboard';
const nameAlias = 'create-dashboard';
export class CreateDashboardExtension {
    constructor(dashboardControl) {
        this.dashboardControl = dashboardControl;
        this.name = name;
        this.showCreateNewDashboard = () => {
            var extension = this.dashboardControl.findExtension('toolbox');
            if (extension) {
                extension.selectMenuItem(this._newDashboardMenuItem);
            }
        };
        this._createNewDashboard = (dataSources, dashboardName) => {
            var toolboxExtension = this.dashboardControl.findExtension('toolbox');
            if (toolboxExtension) {
                toolboxExtension.menuVisible(false);
            }
            let createDashboardCallback = () => {
                var dashboardPropotype = new Dashboard({});
                dashboardPropotype.title.text(dashboardName);
                dataSources.forEach(dataSource => {
                    var newDataSource = Dashboard._createDataSource(new ModelSerializer({ useRefs: false }).serialize(dataSource), new ModelSerializer());
                    dashboardPropotype.dataSources.push(newDataSource);
                });
                this.performCreateDashboard(dashboardName, dashboardPropotype.getJSON());
            };
            let saveExtension = this.dashboardControl.findExtension('saveDashboard');
            if (saveExtension) {
                saveExtension.ensureDashboardSaved(createDashboardCallback);
            }
            else {
                createDashboardCallback();
            }
        };
        var accessibleDataSourcesExtension = ko.computed(() => this.dashboardControl.findExtension('availableDataSources'));
        var clickHandler = () => {
            this._newDashboardMenuItem.template = 'dx-dashboard-form-new';
            this._newDashboardMenuItem.data = new CreateNewDashboardViewModel(accessibleDataSourcesExtension, this._createNewDashboard);
        };
        this._newDashboardMenuItem = new DashboardMenuItem(nameAlias, 'DashboardWebStringId.DashboardMenuNew', 105, 78, () => clickHandler());
    }
    start() {
        var extension = this.dashboardControl.findExtension('toolbox');
        if (extension) {
            extension.menuItems.push(this._newDashboardMenuItem);
        }
        this.dashboardControl._emptyControlTemplates.push({
            name: 'dx-dashboard-add-new-dashboard',
            data: {
                isDesignMode: this.dashboardControl.isDesignMode,
                showCreateNewDashboard: this.showCreateNewDashboard
            }
        });
    }
    stop() {
        var extension = this.dashboardControl.findExtension('toolbox');
        if (extension) {
            extension.menuItems.remove(this._newDashboardMenuItem);
        }
        var template = this.dashboardControl._emptyControlTemplates().filter(temlp => temlp.name === 'dx-dashboard-add-new-dashboard')[0];
        if (template) {
            this.dashboardControl._emptyControlTemplates.remove(template);
        }
    }
    performCreateDashboard(dashboardName, dashboardJson) {
        this.dashboardControl.notificationController.showState(getLocalizationById('DashboardWebStringId.Notification.DashboardLoading'));
        return this.dashboardControl.remoteService.postToServer(this.dashboardControl._endpointCollection.dashboardUrls.GetDashboardsAction, { name: dashboardName, dashboard: dashboardJson })
            .then((result, status, query) => {
            this.dashboardControl.initializeDashboard(result['dashboardId'], result['dashboard'].Dashboard);
        }, request => {
            this.dashboardControl.notificationController.showError(getLocalizationById('DashboardWebStringId.Notification.NewDashboardCannotBeCreated'), request);
        });
    }
}
designerExtensions[nameAlias] = (dashboardControl, options) => new CreateDashboardExtension(dashboardControl);
extensionNameMap[nameAlias] = name;
class CreateNewDashboardViewModel {
    constructor(accessibleDataSourcesExtension, _newDashboardCallback) {
        this.accessibleDataSourcesExtension = accessibleDataSourcesExtension;
        this._newDashboardCallback = _newDashboardCallback;
        this.dashboardName = ko.observable();
        this.selectionMode = 'single';
        this.createNewDashboard = () => {
            this._newDashboardCallback(this.accessibleDataSourcesExtension().selectedDataSources(), this.dashboardName());
        };
        this.dashboardName(getLocalizationById('DashboardWebStringId.NewDashboard'));
        this.createNewDashboardDisabled = ko.computed(() => {
            return !this.dashboardName() || !this.accessibleDataSourcesExtension() || this.accessibleDataSourcesExtension().selectedDataSources().length === 0;
        });
    }
}
