﻿/**
* DevExpress Dashboard (_bubble-map-item-surface.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import * as ko from 'knockout';
import { DataField } from '../../../model/data-sources/_data-field';
import { BubleMapItemPropertiesComposer } from '../properties-composers/_bubble-map-item-properties-composer';
import { DataItemCollectionSurface } from '../sections/_data-item-collection-surface';
import { SingleDataItemSurface } from '../sections/_single-data-item-surface';
import { SectionDescriptors, surfaceItemsFactory } from '../_section-descriptors';
import { DataDashboardItemSurface } from './_base-item-surface';
export class BubbleMapItemSurface extends DataDashboardItemSurface {
    constructor(dashboardItem, dashboardModel, dataSourceBrowser, notificationController) {
        super(dashboardItem, dashboardModel, dataSourceBrowser, notificationController);
    }
    fillSections() {
        this.dataSections.push(new SingleDataItemSurface({
            itemSurface: this,
            sectionInfo: SectionDescriptors.Latitude,
            fieldConstraint: field => DataField.isNumeric(field),
            warning: ko.computed(() => !this.dashboardItem.latitude() && !!(this.dashboardItem.longitude() || this.dashboardItem.weight() || this.dashboardItem.color() || this.dashboardItem.tooltipDimensions().length || this.dashboardItem.tooltipMeasures().length))
        }));
        this.dataSections.push(new SingleDataItemSurface({
            itemSurface: this,
            sectionInfo: SectionDescriptors.Longitude,
            fieldConstraint: field => DataField.isNumeric(field),
            warning: ko.computed(() => !this.dashboardItem.longitude() && !!(this.dashboardItem.latitude() || this.dashboardItem.weight() || this.dashboardItem.color() || this.dashboardItem.tooltipDimensions().length || this.dashboardItem.tooltipMeasures().length))
        }));
        this.dataSections.push(new SingleDataItemSurface({ itemSurface: this, sectionInfo: SectionDescriptors.Weight }));
        this.dataSections.push(new SingleDataItemSurface({
            itemSurface: this,
            sectionInfo: SectionDescriptors.Color,
            warning: ko.computed(() => !this.dashboardItem.color() && !!(this.dashboardItem.latitude() || this.dashboardItem.longitude() || this.dashboardItem.weight() || this.dashboardItem.tooltipDimensions().length || this.dashboardItem.tooltipMeasures().length))
        }));
        this.dataSections.push(new DataItemCollectionSurface(this, SectionDescriptors.TooltipDimensions));
        this.dataSections.push(new DataItemCollectionSurface(this, SectionDescriptors.TooltipMeasures));
    }
    getPropertiesComposer() {
        return new BubleMapItemPropertiesComposer(this._dashboardItemCustomization);
    }
}
surfaceItemsFactory.register('BubbleMap', BubbleMapItemSurface);
