﻿/**
* DevExpress Dashboard (_toolbar-view-model.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import * as ko from 'knockout';
import { toolboxConstants } from '../toolbox-extension/_toolbox-extension';
export let toolbarAnimationTime = 500;
export class ToolbarKoViewModel {
    get height() { return this._visible() ? undefined : 0; }
    constructor(getContainer, toolbarInitialized) {
        this._visible = ko.observable(false);
        this._popupDisposables = [];
        this.left = ko.observable(0);
        this.toolbarItems = ko.observableArray([]);
        this._getContainer = getContainer;
        this._toolbarInitialized = toolbarInitialized;
        this.koToolbarOptions = this._getKoToolbarOptions();
    }
    getKoPopupOptions() {
        let container = this._getContainer();
        let popupOptions = {
            container: container,
            position: {
                my: 'left top',
                at: 'left top',
                of: container,
                collision: 'flipfit',
                boundary: container,
                boundaryOffset: {
                    y: 0
                }
            },
            animation: {
                show: { type: 'slide', from: { top: -toolboxConstants.menuTitleHeight }, to: { top: 0 }, duration: toolbarAnimationTime },
                hide: { type: 'slide', from: { top: 0 }, to: { top: -toolboxConstants.menuTitleHeight }, duration: toolbarAnimationTime }
            },
            onShowing: (args) => {
                let setWidth = () => args.component.option('width', container.getBoundingClientRect().width - this.left());
                this._popupDisposables.push(this.left.subscribe(setWidth));
                let resizeObserver = new ResizeObserver(entries => entries[0] && setWidth());
                resizeObserver.observe(container);
                this._popupDisposables.push({ dispose: () => resizeObserver.unobserve(container) });
                setWidth();
            },
            onHidden: () => {
                this._popupDisposables.forEach(d => d.dispose());
                this._popupDisposables = [];
            },
            focusStateEnabled: false,
            showCloseButton: false,
            dragEnabled: false,
            height: 'auto',
            shading: false,
            wrapperAttr: {
                class: 'dx-dashboard-toolbar-extension'
            },
            showTitle: false
        };
        let koPopupOptions = {
            ...popupOptions,
            visible: this._visible
        };
        let position = koPopupOptions.position;
        let positionOffset = position.boundaryOffset;
        position.boundaryOffset = {
            ...positionOffset,
            x: this.left
        };
        return koPopupOptions;
    }
    _getKoToolbarOptions() {
        let toolbarOptions = {
            onInitialized: (args) => {
                this._toolbarInitialized();
            }
        };
        let koToolbarOptions = {
            ...toolbarOptions,
            items: this.toolbarItems
        };
        return koToolbarOptions;
    }
    setToolbarItems(items) {
        this.toolbarItems(items);
    }
    showPanel() {
        this._visible(true);
    }
    hidePanel() {
        this._visible(false);
    }
}
