﻿/**
* DevExpress Dashboard (_datasource-chooser-surface.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { getTemplate } from '@devexpress/analytics-core/analytics-utils';
import * as ko from 'knockout';
import { FieldChooserList, SliderController } from './_field-chooser-surface';
export class DataSourceChooserController extends SliderController {
    constructor(params) {
        super({ startPath: ko.observable(''), dataSourceBrowser: params.dataSourceBrowser, selectedField: ko.observable(), filter: undefined });
        this.toDispose(this.selectedField.subscribe(field => {
            params.dataSource(field && field['dataSourceName']);
            params.dataMember(field && field['dataMemberName']);
        }));
        var updatingWrapper = (func) => {
            this.lists()[0].ready(false);
            func();
            setTimeout(() => {
                this.lists().forEach(list => list.ready(true));
            }, 1);
        };
        var updateSelection = () => {
            if (params.dataSource() && params.dataMember()) {
                updatingWrapper(() => {
                    var newList = new FieldChooserList(this, params.dataSource(), [params.dataSource()], this.selectedField);
                    newList.index(1);
                    this.lists.push(newList);
                    this.lists().forEach((list, index) => list.index(index - this.lists().length + 1));
                });
            }
            this.selectedField({ dataMember: ko.observable(params.dataMember() || params.dataSource()), dataSourceName: params.dataSource(), dataMemberName: params.dataMember() });
            if (params.dataSource() && !params.dataMember()) {
                this.lists()[0].scrollToSelectedItem();
            }
        };
        if (ko.isSubscribable(params.active)) {
            this.toDispose(params.active.subscribe(newVal => {
                if (newVal) {
                    updateSelection();
                }
                else {
                    this.selectedField(undefined);
                    updatingWrapper(() => {
                        this.lists([this.lists()[0]]);
                        this.lists()[0].index(0);
                    });
                }
            }));
        }
        updateSelection();
    }
}
export const _registerDxDatasourceChooser = () => {
    const componentName = 'dx-datasource-chooser';
    const templateName = 'dx-datasource-chooser-slider';
    if (ko.components.isRegistered(componentName))
        return;
    ko.components.register(componentName, {
        viewModel: {
            createViewModel: function (params, componentInfo) {
                var viewModel = new DataSourceChooserController(params);
                const disposeCallback = () => {
                    viewModel.dispose();
                    ko.utils.domNodeDisposal.removeDisposeCallback(componentInfo.element, disposeCallback);
                };
                ko.utils.domNodeDisposal.addDisposeCallback(componentInfo.element, disposeCallback);
                return viewModel;
            }
        },
        template: getTemplate(templateName)
    });
};
