﻿/**
* DevExpress Dashboard (color-scheme-entry.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { ModelSerializer, deserializeArray } from '@devexpress/analytics-core/analytics-utils';
import * as ko from 'knockout';
import { getLocalizationById } from '../../data/localization/_default';
import { TypedSerializableModel } from '../serializable-model';
import { ColorSchemeDefinition } from './color-scheme-definition';
import { DimensionKey } from './dimension-key';
import { MeasureDefinition } from './measure-definition';
import { colorSchemeEntrySerializationInfo } from './metadata/_color-scheme-entry';
export class ColorSchemeEntry extends TypedSerializableModel {
    get custom() {
        return true;
    }
    get definition() {
        if (!this._definition) {
            this._definition = new ColorSchemeDefinition(this.dataSource(), this.dataMember(), this.dimensionKeys().map(key => { return key.definition; }), !!this.measureKeys().length, this.itemComponentName, this.name);
        }
        return this._definition;
    }
    constructor(modelJson = {}, serializer = new ModelSerializer(), itemComponentName = '', name = '') {
        super(modelJson, serializer, colorSchemeEntrySerializationInfo);
        this.itemComponentName = itemComponentName;
        this.name = name;
        this.measureKeys = ko.observableArray();
        this.dimensionKeys = ko.observableArray();
        this.measureKeys = deserializeArray(modelJson.MeasureKey, (item) => new MeasureDefinition(item, serializer));
        this.dimensionKeys = deserializeArray(modelJson.DimensionKeys, (item) => new DimensionKey(item, serializer));
        this.addDisposable(this.displayText = ko.computed(() => {
            return this.dimensionKeys()
                .map(key => key.value.value())
                .concat(this.measureKeys().map(key => key.displayText()))
                .join(' | ');
        }));
        this.addDisposable(this.colorText = ko.computed(() => {
            var result = '';
            if (!this.custom) {
                result = getLocalizationById('DashboardStringId.ColorAutoAssigned');
            }
            else {
                if (this.paletteIndex() !== null)
                    result = getLocalizationById('DashboardStringId.ColorPaletteIndex') + this.paletteIndex();
                else if (!!this.color()) {
                    result = getLocalizationById('DashboardWebStringId.ColorScheme.Color') + ' [A=' + (this.color().A * 255).toFixed(0) + ', R=' + this.color().R + ', G=' + this.color().G + ', B=' + this.color().B + ']';
                }
            }
            return result;
        }));
        this.addDisposable(this.componentName = ko.computed(() => {
            return this.displayText() + this.dataSource() + this.dataMember();
        }));
    }
    dispose() {
        super.dispose();
        this.disposeObservableArray(this.measureKeys);
        this.disposeObservableArray(this.dimensionKeys);
    }
    getInfo() {
        return colorSchemeEntrySerializationInfo;
    }
    clone() {
        var clonedEntry = new ColorSchemeEntry();
        clonedEntry.itemComponentName = this.itemComponentName;
        clonedEntry.name = this.name;
        clonedEntry.dataMember(this.dataMember());
        clonedEntry.dataSource(this.dataSource());
        clonedEntry.dimensionKeys(this.dimensionKeys());
        clonedEntry.measureKeys(this.measureKeys());
        clonedEntry.paletteIndex(this.paletteIndex());
        clonedEntry.color(this.color());
        return clonedEntry;
    }
    equals(entry) {
        if (this.dimensionKeys().length !== entry.dimensionKeys().length)
            return false;
        for (var i = 0; i < this.dimensionKeys().length; i++) {
            if (this.dimensionKeys()[i].value.value() !== entry.dimensionKeys()[i].value.value()) {
                return false;
            }
        }
        if (this.definition.colorByMeasures) {
            if (this.measureKeys().length !== entry.measureKeys().length)
                return false;
            for (var i = 0; i < this.measureKeys().length; i++) {
                if (this.measureKeys()[i].dataMember() !== entry.measureKeys()[i].dataMember() || this.measureKeys()[i].summaryType() !== entry.measureKeys()[i].summaryType())
                    return false;
            }
        }
        return true;
    }
    getUniqueNamePrefix() {
        return super._getUniqueNamePrefix();
    }
    _getDefaultItemType() {
        return 'Entry';
    }
}
export class AutoColorSchemeEntry extends ColorSchemeEntry {
    constructor(modelJson = {}, serializer = new ModelSerializer(), componentName, name) {
        super(modelJson, serializer, componentName, name);
    }
    get custom() {
        return false;
    }
}
