﻿/**
* DevExpress Dashboard (difference-calculation.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { ModelSerializer } from '@devexpress/analytics-core/analytics-utils';
import { calculationsTypesMap, MeasureCalculation } from './measure-calculation';
import { differenceCalculationSerializationsInfo } from './metadata/_difference-calculation';
export class DifferenceCalculation extends MeasureCalculation {
    static getLookupShiftExpression(target) {
        switch (target) {
            case 'Previous':
                return '-1';
            case 'Next':
                return '1';
            case 'First':
                return 'First()';
            case 'Last':
                return 'First()';
            default:
                throw new Error('Unsupported Target');
        }
    }
    get name() {
        return 'DashboardWebStringId.Calculations.DifferenceCalculation';
    }
    constructor(modelJson = {}, serializer = new ModelSerializer()) {
        super(modelJson, serializer);
    }
    get lookupShiftExpression() {
        return DifferenceCalculation.getLookupShiftExpression(this.target());
    }
    getInfo() {
        return differenceCalculationSerializationsInfo;
    }
    _createInstance() {
        return new DifferenceCalculation();
    }
    _getExpression(argument) {
        switch (this.differenceType()) {
            case 'Absolute':
                return `${argument} - Lookup(${argument}, ${this.lookupShiftExpression})`;
            case 'Percentage':
                return `ToDouble(${argument} - Lookup(${argument}, ${this.lookupShiftExpression})) / Lookup(${argument}, ${this.lookupShiftExpression})`;
            default:
                throw new Error('Unexpected DifferenceType');
        }
    }
}
calculationsTypesMap['DifferenceCalculation'] = DifferenceCalculation;
