﻿/**
* DevExpress Dashboard (rank-calculation.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { ModelSerializer } from '@devexpress/analytics-core/analytics-utils';
import { calculationsTypesMap, MeasureCalculation } from './measure-calculation';
import { rankCalculationSerializationsInfo } from './metadata/_rank-calculation';
export class RankCalculation extends MeasureCalculation {
    static _getRankFunction(rankType) {
        switch (rankType) {
            case 'Competition':
                return 'RankCompetition';
            case 'Unique':
                return 'RankUnique';
            case 'Dense':
                return 'RankDense';
            case 'Modified':
                return 'RankModified';
            case 'Percentile':
                return 'RankPercentile';
            default:
                throw new Error('Unsupported RankType');
        }
    }
    get name() {
        return 'DashboardWebStringId.Calculations.Rank';
    }
    constructor(modelJson = {}, serializer = new ModelSerializer()) {
        super(modelJson, serializer);
    }
    getInfo() {
        return rankCalculationSerializationsInfo;
    }
    _createInstance() {
        return new RankCalculation();
    }
    _getExpression(argument) {
        var order = this.rankOrder() == 'Ascending' ? 'asc' : 'desc';
        return `${RankCalculation._getRankFunction(this.rankType())}(${argument}, '${order}')`;
    }
}
calculationsTypesMap['Rank'] = RankCalculation;
