﻿/**
* DevExpress Dashboard (custom-item.d.ts)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { ISerializationInfoArray, ModelSerializer } from '@devexpress/analytics-core/analytics-utils';
import * as ko from 'knockout';
import { SectionOptions } from '../../../designer/accordion-tab-options';
import { ColorSchemeEntry } from '../../colorization/color-scheme-entry';
import { CustomPropertyMetadata } from '../../custom-properties/custom-properties-metadata';
import { DataItemLink } from '../../data-item/data-item';
import { MeasureCalculationWindowDefinition } from '../../data-item/window-definition/measure-calc-window-definition';
import { IDashboardSerializationInfo } from '../../metadata/_base-metadata';
import { DataDashboardItem } from '../data-dashboard-item';
import { DashboardItemColoringOptions } from '../options/coloring-options';
import { ICustomItemBinding, ICustomItemBindingValue } from './binding';
import { ICustomDataRow } from './data-row';
import { ICustomItemMetaData } from './meta';
import { ICustomItemProperty } from './property';
import { SliceTable } from './slice-table';
export declare class CustomItem extends DataDashboardItem {
    private _meta;
    static ItemType: string;
    static getPropertyInfo(p: ICustomItemProperty): IDashboardSerializationInfo;
    static _getBindingLinkName(propertyName: string): string;
    private static _getSerializationsInfo;
    private static _hasInteractivityTargets;
    private static _hasColoringDimensions;
    private static _hasColoringMeasures;
    private static _isFilterAllowed;
    private static _isDrillDownAllowed;
    private static _getInfo;
    colorScheme: ko.ObservableArray<ColorSchemeEntry>;
    coloringOptions: DashboardItemColoringOptions;
    customItemType: ko.Observable<string>;
    sliceTables: ko.ObservableArray<SliceTable>;
    interactivityOptions: any;
    interactivityTargets: ko.ObservableArray<DataItemLink>;
    coloringDimensions: ko.ObservableArray<DataItemLink>;
    coloringMeasures: ko.ObservableArray<DataItemLink>;
    customMetadata: {};
    constructor(_meta: ICustomItemMetaData, modelJson?: any, serializer?: ModelSerializer);
    _clearBindings(): void;
    getBindingValue(propertyName: string, index?: number): Array<ICustomItemBindingValue>;
    iterateData(action: (item: ICustomDataRow) => any, sliceTableName?: string): void;
    protected _getDefaultItemType(): string;
    private _getSliceTable;
    _getAllSelectionValues(activeDimensions: Array<string>): any[];
    private getSlice;
    private _getStorage;
    private _getValidIds;
    private _getColor;
    private _getColorMeasureId;
    private _getMeasureValue;
    private _getMeasureDisplayText;
    private _getDimensionUniqueValue;
    private _getDimensionValue;
    private _getDimensionDisplayText;
    private _format;
    private _getServerText;
    private _getSpecialValue;
    private _getValueBySliceKey;
    get customBindings(): Array<ICustomItemBinding>;
    get legacyCustomProperties(): Array<ICustomItemProperty>;
    get customInteractivity(): any;
    get _optionsPanelSections(): Array<SectionOptions>;
    protected _getMasterFilterMode(): string;
    protected _getDrillDownEnabled(): boolean;
    protected _getIgnoreMasterFilter(): boolean;
    protected _getInteractivityDimensionLinks(): DataItemLink[];
    protected _getIsVisualInteractivitySupported(): boolean;
    isDrillDownAllowed(): boolean;
    isFilterAllowed(): boolean;
    _isInteractivityAllowed(): boolean;
    protected _getCanColorByMeasures(): boolean;
    protected _getCanColorByDimensions(): boolean;
    _getColorizableDataItemsInfo(): Array<{
        items: Array<DataItemLink>;
        prefixId: string;
    }>;
    _getDefaultCalculationWindowDefinition(): MeasureCalculationWindowDefinition;
    getInfo(): ISerializationInfoArray;
    protected _getInfoCore(): IDashboardSerializationInfo[];
    static _getCustomPropertiesSerializationInfo(customItemType: string, dynamicCustomPropertiesMetadata: Array<CustomPropertyMetadata>): IDashboardSerializationInfo;
}
