﻿/**
* DevExpress Dashboard (dashboard-item.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { ModelSerializer } from '@devexpress/analytics-core/analytics-utils';
import * as ko from 'knockout';
import { contentType } from '../../data/_common';
import { deepExtend } from '../../data/_jquery-helpers';
import { getCustomPropertiesSerializationInfo } from '../custom-properties/_custom-properties-utils';
import { PropertyCategory } from '../metadata/_base-metadata';
import { TypedSerializableModel } from '../serializable-model';
import { PaneContentHolder } from './_pane-content-holder';
import { commonInteractivityOptions } from './options/metadata/_interactivity-options';
export class DashboardItem extends TypedSerializableModel {
    static _getCommonItemType(itemType) {
        var commonItemType = itemType.toUpperCase();
        if (commonItemType === 'TEXTBOX') {
            return 'TEXT';
        }
        return commonItemType;
    }
    getUniqueNamePrefix() {
        return super._getUniqueNamePrefix() + 'DashboardItem';
    }
    _removeProperties() {
        Object.keys(this).forEach((propertyName) => {
            if (propertyName !== 'componentName')
                delete this[propertyName];
        });
    }
    _onDispose() { }
    dispose() {
        super.dispose();
        this.customProperties?.dispose();
        this._onDispose();
        this._removeProperties();
        this.disposed = true;
    }
    get _caption() {
        var name = this.name(), layerName = '';
        if (this._getLayersCount() > 0) {
            layerName = this._getLayerName();
            name += !!name && !!layerName ? ' - ' : '';
        }
        return name + layerName;
    }
    get _elementId() {
        const id = 'dx-dashboard-item-' + this.componentName() + '-' + this.name();
        return id.replace(/\W+/g, '');
    }
    _updateContent() {
        if (this._paneContentHolder.valid && this._paneContentHolder.valid() && !this._errorState()) {
            var content = deepExtend({}, this._paneContentHolder.getContent(PropertyCategory.Data) || {});
            this._updateContentViewModel(content);
            this._updateContentData(content);
            this._extendContentState(content);
            this._serverContent(content);
        }
        else {
            this._serverContent(null);
        }
    }
    constructor(dashboardItemJSON = {}, serializer = new ModelSerializer(), info = undefined) {
        super(dashboardItemJSON, serializer, info);
        this._useNeutralFilterMode = ko.observable(false);
        this._uiState = ko.observable('live');
        this._errorState = ko.observable(null);
        this._viewerItemCreated = ko.observable(false);
        this._allowMultiselection = ko.observable(false);
        this._serverContent = ko.observable(null);
        this.addDisposable(this._paneContentHolder = new PaneContentHolder());
        this.addDisposable(this._actions = ko.computed(() => []));
        this.addDisposable(this._state = ko.computed(() => null));
        this.addDisposable(this._dataQueryParams = ko.computed(() => null));
        this.addDisposable(this._paneContentHolder.valid.subscribe(() => this._updateContent()));
        this.addDisposable(this._errorState.subscribe(() => this._updateContent()));
    }
    _isInteractivityAllowed() {
        return !!this[commonInteractivityOptions.propertyName];
    }
    _shouldSuspendOnMasterFilterSet() {
        return !this._useNeutralFilterMode();
    }
    _getLayersCount() {
        return 0;
    }
    _getLayerName() {
        return '';
    }
    _updateContentViewModel(content) {
        content.CaptionViewModel = content.CaptionViewModel || {};
        content.CaptionViewModel.Caption = this.name();
        content.CaptionViewModel.Text = this.name();
        content.CaptionViewModel.ShowCaption = this.showCaption();
        content.ViewModel = content.ViewModel || {};
        content.ViewModel.Caption = this.name();
        content.ViewModel.ShowCaption = this.showCaption();
        content.ParentContainer = this.parentContainer();
        content.forceUpdateViewModel = this._paneContentHolder.lastChangeReason === PropertyCategory.ViewModel;
    }
    _updateContentData(content) {
    }
    _updateDataQueryParams(params) {
    }
    _validateSelectionByData(selection) {
    }
    _extendContentState(content) {
    }
    _getDisplayFilterValues(limitCount) {
        return undefined;
    }
    _getDisplayFilterValuesExternal() {
    }
    _getServerContent() {
        let content = this._serverContent();
        var contentCopy = content ? deepExtend({}, content) : undefined;
        if (contentCopy) {
            this._extendContentState(contentCopy);
        }
        return contentCopy;
    }
    _getFullServerContent() {
        let serverContent = this._getServerContent();
        return !!serverContent ? { ...serverContent, ContentType: contentType.fullContent } : serverContent;
    }
    _subcribeServerContent(handler) {
        return this._serverContent.subscribe(() => {
            handler(this._getServerContent());
        });
    }
    _getContentCategories() {
        return [PropertyCategory.Initialize];
    }
    _getDataQueryParams() {
        var params = this._dataQueryParams.peek();
        this._updateDataQueryParams(params);
        return params || {};
    }
    _subcribeDataQueryParams(handler) {
        return this._dataQueryParams.subscribe(() => handler(this._getDataQueryParams()));
    }
    _getExportingSelection() {
    }
    _setState(parameter) {
    }
    getInfo() {
        return this._getInfoCore().concat(getCustomPropertiesSerializationInfo(this));
    }
}
