﻿/**
* DevExpress Dashboard (combo-box-item.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { ModelSerializer } from '@devexpress/analytics-core/analytics-utils';
import { HashsetWrapper } from '../../../data/_hashset-wrapper';
import { itemTypesMap } from '../../serializable-model';
import { FilterElementItemBase } from './filter-element-item-base';
import { comboBoxDashboardItemSerializationsInfo } from './metadata/_combo-box-item';
export class ComboBoxItem extends FilterElementItemBase {
    constructor(dashboardItemJSON = {}, serializer = new ModelSerializer()) {
        super(dashboardItemJSON, serializer);
    }
    _getInfoCore() {
        return comboBoxDashboardItemSerializationsInfo;
    }
    _getDefaultItemType() {
        return 'ComboBox';
    }
    _getUpdatedSelectionData(selection) {
        if (!this._multiData) {
            return selection;
        }
        var currentSelection = this._selectionValues();
        if (!this._useNeutralFilterMode() &&
            this._isMultiselectable() &&
            !!currentSelection &&
            this._unselectedValues()) {
            this._updateUnselectedValues(selection, currentSelection);
        }
        if (!!selection) {
            var hashset = new HashsetWrapper(selection);
            var allValues = this._getAllSelectionValues(this._interactivityDimensions.map(d => d.uniqueName()));
            return hashset.getIntersection(allValues);
        }
        return selection;
    }
    _setSelectionData(selection, forceSetSelection = false) {
        super._setSelectionData(this._getUpdatedSelectionData(selection), forceSetSelection);
    }
    _correctSelectionValues(allValues, currentSelection) {
        if (!this._isMultiselectable()) {
            return super._correctSelectionValues(allValues, currentSelection);
        }
        if (this._useNeutralFilterMode()) {
            return allValues.length <= currentSelection.length
                ? allValues
                : currentSelection;
        }
        var unknownSelection = allValues.filter(x => (!this._arrayContains(currentSelection, x) &&
            !this._arrayContains(this._unselectedValues(), x)));
        return currentSelection.concat(unknownSelection);
    }
    _allowAllValue() { return this._isMultiselectable() || this.showAllValue(); }
    _isMultiselectable() { return this.comboBoxType() === 'Checked'; }
}
itemTypesMap['ComboBox'] = { type: ComboBoxItem, groupName: 'filter', title: 'DashboardStringId.DefaultNameComboBoxItem', index: 320 };
