﻿/**
* DevExpress Dashboard (map-item.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { deserializeArray, ModelSerializer } from '@devexpress/analytics-core/analytics-utils';
import * as ko from 'knockout';
import { AcceptableShapingType, DataItemLink } from '../../data-item/data-item';
import { areArraysEqual } from '../../internal/_array-utils';
import { collectionItemType } from '../../internal/_utils';
import { PropertyCategory } from '../../metadata/_base-metadata';
import { DataDashboardItem } from '../data-dashboard-item';
import { shapeData } from './metadata/_custom-shape-file-data';
import { area, mapDashboardItemSerializationsInfo, tooltipMeasures, viewport } from './metadata/_map-item';
export class MapItem extends DataDashboardItem {
    constructor(modelJson = {}, serializer = new ModelSerializer()) {
        super(modelJson, serializer);
        this.tooltipMeasures = ko.observableArray([]);
        this._isGeometryChanged = false;
        this._shapeFilesAttributeNameList = ko.observableArray();
        this._initialExtentChanged = ko.observable(false);
        this._isGeometryChangedCallback = () => {
            this._isGeometryChanged = true;
            this.viewport.createViewerPaddings(true);
        };
        this.__tooltipMeasures = deserializeArray(modelJson.TooltipMeasures, (item) => new DataItemLink(this, item, serializer));
        this._subscribeDataItemLinkArrays(tooltipMeasures);
        this.addDisposable(this.customShapefile);
        this.addDisposable(this.__tooltipMeasures.subscribe((links) => {
            this._setLinkCollectionAcceptableShapingType(links, AcceptableShapingType.String);
        }));
        this._setLinkCollectionAcceptableShapingType(this.__tooltipMeasures(), AcceptableShapingType.String);
        if (!modelJson[viewport.modelName] &&
            !modelJson[area.modelName] &&
            !modelJson[shapeData.modelName]) {
            this.viewport._set({
                'TopLatitude': 83.64513000000001,
                'BottomLatitude': -55.61182999999996,
                'CenterPointLatitude': 44.08908074480383,
                'CenterPointLongitude': 1.4210854715202004e-13,
                'LeftLongitude': -179.99999999999997,
                'RightLongitude': 180
            });
        }
        this._supportedUIStates(['error', 'loading']);
    }
    _clearBindings() {
        super._clearBindings();
        this.disposeObservableArray(this.__tooltipMeasures);
        this.__tooltipMeasures.removeAll();
    }
    _getInfoCore() {
        return mapDashboardItemSerializationsInfo;
    }
    _isCalculationSupported() {
        return false;
    }
    _getMasterFilterMode() { return this.interactivityOptions.masterFilterMode(); }
    _getDrillDownEnabled() { return false; }
    _getIgnoreMasterFilter() { return this.interactivityOptions.ignoreMasterFilters(); }
    _setClientState(clientState) {
        super._setClientState(clientState);
    }
    _getContentCategories() {
        return super._getContentCategories().concat([PropertyCategory.Map]);
    }
    _updateContentViewModel(content) {
        super._updateContentViewModel(content);
        var mapContent = this._paneContentHolder.getContent(PropertyCategory.Map);
        if (!areArraysEqual(this._shapeFilesAttributeNameList(), mapContent.MapAttributesNames)) {
            this._shapeFilesAttributeNameList(mapContent.MapAttributesNames);
        }
        if (mapContent && mapContent.ViewModel) {
            content.ViewModel.MapItems = mapContent.ViewModel.MapItems;
            content.ViewModel.ShapeTitleAttributeName = mapContent.ViewModel.ShapeTitleAttributeName;
            content.FullViewport = mapContent.FullViewport;
            if (this._isGeometryChanged) {
                this.viewport._set(mapContent.FullViewport);
                this._isGeometryChanged = false;
            }
        }
        content.ViewModel.Viewport = this.viewport._createViewModel();
    }
    _isSortingEnabled() {
        return false;
    }
}
__decorate([
    collectionItemType('TooltipMeasure')
], MapItem.prototype, "__tooltipMeasures", void 0);
