﻿/**
* DevExpress Dashboard (_cacheable.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
let globalCache = {};
export function resetGlobalSizeCache() {
    globalCache = {};
}
export function cacheable(cacheKey) {
    return (target, propertyKey, descriptor) => {
        if (typeof target !== 'function') {
            throw Error('The cacheable decorator can be applied only for static methods');
        }
        return {
            value: (...args) => {
                if (!globalCache[cacheKey]) {
                    globalCache[cacheKey] = {};
                }
                var functionCache = globalCache[cacheKey];
                var argsKey = JSON.stringify(args);
                if (functionCache.hasOwnProperty(argsKey)) {
                    return functionCache[argsKey];
                }
                else {
                    var result = descriptor.value.apply(this, args);
                    functionCache[argsKey] = result;
                    return result;
                }
            }
        };
    };
}
