﻿/**
* DevExpress Dashboard (_viewer-item-factory.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { dataGridItem } from './viewer-items/data-grid-item/_data-grid-item';
import { comboBoxFilterElement } from './viewer-items/filter-items/_combo-box-element';
import { dateFilterElement } from './viewer-items/filter-items/_date-filter-element';
import { listFilterElement } from './viewer-items/filter-items/_list-element';
import { treeViewFilterElement } from './viewer-items/filter-items/_tree-element';
import { pivotGridItem } from './viewer-items/pivot-grid-item/_pivot-grid-item';
import { rangeSelectorItem } from './viewer-items/range-selector-item/_range-selector-item';
import { boundImageItem } from './viewer-items/_bound-image-item';
import { bubbleMapItem } from './viewer-items/_bubble-map-item';
import { cardsItem } from './viewer-items/_cards-item';
import { chartItem } from './viewer-items/_chart-item';
import { choroplethMapItem } from './viewer-items/_choropleth-map-item';
import { customFakeItem } from './viewer-items/_custom-fake-item';
import { gaugesItem } from './viewer-items/_gauges-item';
import { geoPointMapItem } from './viewer-items/_geo-point-map-item';
import { groupItem } from './viewer-items/_group-item';
import { imageItem } from './viewer-items/_image-item';
import { pieItem } from './viewer-items/_pie-item';
import { pieMapItem } from './viewer-items/_pie-map-item';
import { tabPageItem } from './viewer-items/_tab-page-item';
import { textItem } from './viewer-items/_text-item';
import { treemapItem } from './viewer-items/_treemap-item';
import { types } from './_viewer-item-types';
export class ViewerItemFactory {
    createItem(container, options) {
        switch (options.Type) {
            case types.group:
                return new groupItem(container, options);
            case types.tabPage:
                return new tabPageItem(container, options);
            case types.rangeFilter:
                return new rangeSelectorItem(container, options);
            case types.chart:
            case types.scatter:
                return new chartItem(container, options);
            case types.pie:
                return new pieItem(container, options);
            case types.card:
                return new cardsItem(container, options);
            case types.grid:
                return new dataGridItem(container, options);
            case types.pivot:
                return new pivotGridItem(container, options);
            case types.gauge:
                return new gaugesItem(container, options);
            case types.text:
                return new textItem(container, options);
            case types.image:
                return new imageItem(container, options);
            case types.boundImage:
                return new boundImageItem(container, options);
            case types.map:
            case types.choroplethMap:
                return new choroplethMapItem(container, options);
            case types.geoPointMap:
                return new geoPointMapItem(container, options);
            case types.bubbleMap:
                return new bubbleMapItem(container, options);
            case types.pieMap:
                return new pieMapItem(container, options);
            case types.treemap:
                return new treemapItem(container, options);
            case types.comboBox:
                return new comboBoxFilterElement(container, options);
            case types.listBox:
                return new listFilterElement(container, options);
            case types.treeView:
                return new treeViewFilterElement(container, options);
            case types.dateFilter:
                return new dateFilterElement(container, options);
            case types.custom:
                return new customFakeItem(container, options);
            default:
                return {};
        }
    }
}
export let defaultViewerItemFactory = new ViewerItemFactory();
