﻿/**
* DevExpress Dashboard (_filter-icon-tooptip.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import * as $ from 'jquery';
import { formatFilterValue } from '../../data/_formatter';
import { RenderHelper } from '../widgets/_render-helper';
export let titleTooltipClasses = {
    root: 'dx-dashboard-title-tooltip',
    list: 'dx-dashboard-title-tooltip-list',
    listText: 'dx-dashboard-title-tooltip-list-text',
    subList: 'dx-dashboard-title-tooltip-sublist',
    subListItem: 'dx-dashboard-title-tooltip-sublist-item'
};
export class FilterIconTooltip {
    static getFilterValuesString(masterFilterValues) {
        let result = '';
        masterFilterValues.forEach((argument, i) => {
            if (argument.Values && argument.Values.length) {
                let subCategoryValues = argument.Values?.flatMap(i => i.Value).join(', ');
                result += `${i !== 0 ? ', ' : ''} ${argument.Name} : ${subCategoryValues}`;
            }
        });
        return result;
    }
    static getTooltipContent(contentElement, masterFilterValues) {
        contentElement.addClass(titleTooltipClasses.root);
        let $tooltipList = $.fn.constructor('<ul/>', { 'class': titleTooltipClasses.list });
        masterFilterValues.forEach((argument, i) => {
            let $tooltipListItem = $.fn.constructor('<li/>').appendTo($tooltipList);
            $tooltipListItem.css({ 'padding-top': i == 0 ? 0 : this.FILTER_LIST_SPACE + 'px' });
            RenderHelper.html($.fn.constructor('<div/>', { 'class': titleTooltipClasses.listText }).appendTo($tooltipListItem).get(0), argument.Name, true);
            let $tooltipSublist = $.fn.constructor('<ul/>', { 'class': titleTooltipClasses.subList }).appendTo($tooltipListItem);
            let maxFilterListValues = this._calcMaxFilterListValues(masterFilterValues);
            var maxFilterValues = this._calcMaxFilterValues(maxFilterListValues, masterFilterValues);
            for (var j = 0; j < maxFilterValues; j++) {
                if (j >= argument.Values.length && argument.Truncated
                    || j + 1 == maxFilterValues && (j + 1 < argument.Values.length
                        || argument.Truncated)) {
                    $.fn.constructor('<li/>', { 'class': titleTooltipClasses.subListItem }).appendTo($tooltipSublist).append('...');
                    break;
                }
                if (j >= argument.Values.length) {
                    break;
                }
                RenderHelper.html($.fn.constructor('<li/>', { 'class': titleTooltipClasses.subListItem }).appendTo($tooltipSublist).get(0), formatFilterValue(argument.Values[j]), true);
            }
        });
        return $tooltipList;
    }
    static _calcMaxFilterListValues(masterFilterValues) {
        var that = this, lineHeightString = $.fn.constructor('<div/>', { 'class': titleTooltipClasses.root }).css('line-height'), lineHeightTmp = parseInt(lineHeightString, 10), lineHeight = lineHeightTmp && lineHeightString.length > 2 && lineHeightString.substr(lineHeightString.length - 2) == 'px' ? lineHeightTmp : this.DEFAULT_LINE_HEIGHT, maxHeight = this._calcMaxHeight();
        return Math.floor((maxHeight - (masterFilterValues.length - 1) * this.FILTER_LIST_SPACE) / lineHeight);
    }
    static _calcMaxFilterValues(maxFilterListValues, masterFilterValues) {
        var that = this, MIN_FILTER_VALUES = 4, MAX_FILTER_LIST_VALUES = 100, maxFilterValues, curFilterListValues;
        maxFilterListValues = Math.min(maxFilterListValues, MAX_FILTER_LIST_VALUES);
        for (maxFilterValues = Math.max(maxFilterListValues, MIN_FILTER_VALUES); maxFilterValues >= MIN_FILTER_VALUES; maxFilterValues--) {
            if (maxFilterValues == MIN_FILTER_VALUES) {
                break;
            }
            curFilterListValues = 0;
            $.each(masterFilterValues, function (index, dimensionFilterValues) {
                curFilterListValues += (maxFilterValues < dimensionFilterValues.Values.length ? maxFilterValues : dimensionFilterValues.Values.length) + 1;
                if (curFilterListValues > maxFilterListValues) {
                    return false;
                }
            });
            if (curFilterListValues <= maxFilterListValues) {
                break;
            }
        }
        return maxFilterValues;
    }
    static _calcMaxHeight() {
        return Math.floor($.fn.constructor(window).height() * 0.75);
    }
}
FilterIconTooltip.FILTER_LIST_SPACE = 8;
FilterIconTooltip.DEFAULT_LINE_HEIGHT = 16;
