﻿/**
* DevExpress Dashboard (_base-item.d.ts)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
/// <reference types="jquery" />
/// <reference types="jquery" />
import { DxElement } from '../../common/common-interfaces';
import { DashboardExportFormat } from '../../common/extensions/export-extension';
import { ItemDataAxisName } from '../../data';
import { ContentType } from '../../data/_common';
import { dataControllerBase } from '../../data/data-controllers/_data-controller-base';
import { itemDataTupleValues } from '../../data/item-data/_item-data-tuple';
import { IViewport } from '../../model/internal/_interfaces';
import { ConstraintsEx, ISize } from '../layout/_utils.layout';
import { CaptionToolbar, ItemCaptionToolbarViewOptions } from '../widgets/caption-toolbar/_caption-toolbar-base';
import { DashboardItemCaptionToolbarOptions, ViewerToolbarItem } from '../widgets/caption-toolbar/caption-toolbar-options';
import { MapClientState } from './_map-item';
import { GridClientState } from './data-grid-item/_data-grid-item';
import { IItemLoadingElement } from './elements/_item-loading';
export declare var createDefaultToolbar: (viewerItem: baseItem, container: HTMLElement, controlContainer: HTMLElement, popupContainer: HTMLElement, viewOptions: ItemCaptionToolbarViewOptions) => CaptionToolbar;
export type ViewerItemVisualMode = 'full' | 'caption' | 'content';
export declare abstract class baseItem {
    private _lockCount;
    _isFixedHeight: boolean;
    customHoverEnabled: boolean;
    container: HTMLElement;
    controlContainer: HTMLElement;
    _boundaryContainer: HTMLElement;
    _clearMasterFilterHandler: any;
    _clearSelectionHandler: any;
    _toggleSelectionModeHandler: any;
    _drillUpHandler: any;
    _allowMultiselection: boolean;
    allowMultiselectionChanged: (allowed: any) => void;
    allowLimitDataCallback: () => void;
    dateToString: (date: Date) => string;
    selected: JQuery.Callbacks<Function>;
    clearMasterFilter: JQuery.Callbacks<Function>;
    drillUp: JQuery.Callbacks<Function>;
    contentElementSelection: JQuery.Callbacks<Function>;
    expandValue: JQuery.Callbacks<Function>;
    clientStateUpdate: JQuery.Callbacks<Function>;
    dataRequest: JQuery.Callbacks<Function>;
    itemClick: JQuery.Callbacks<Function>;
    itemHover: JQuery.Callbacks<Function>;
    itemSelectionChanged: JQuery.Callbacks<Function>;
    itemWidgetCreated: JQuery.Callbacks<Function>;
    itemWidgetUpdating: JQuery.Callbacks<Function>;
    itemWidgetUpdated: JQuery.Callbacks<Function>;
    itemWidgetOptionsPrepared: JQuery.Callbacks<Function>;
    itemCaptionToolbarUpdated: JQuery.Callbacks<Function>;
    constraintsUpdated: JQuery.Callbacks<Function>;
    interactivityController: any;
    customSelectionMode: any;
    customTargetAxes: ItemDataAxisName[];
    customDefaultSelectedValues: any[];
    options: ViewerItemOptions;
    customSelectedTuples: itemDataTupleValues[];
    contentRoot: HTMLElement;
    shieldingElement: HTMLElement;
    itemLoadingElement: IItemLoadingElement;
    captionToolbar: CaptionToolbar;
    visualMode: ViewerItemVisualMode;
    protected _hasWidget: boolean;
    get hasWidget(): boolean;
    protected get _captionToolbarSeparatorRequired(): boolean;
    protected get _isBottomFloatingToolbarPosition(): boolean;
    protected get _allowPreview(): boolean;
    protected get _shouldApplySelectionOnInitialRender(): boolean;
    protected get dataController(): dataControllerBase;
    protected set dataController(dataController: dataControllerBase);
    constructor($container: HTMLElement, options: ViewerItemOptions);
    protected _initializeData(newOptions: ViewerItemOptions): void;
    initialDataRequest(): void;
    initialDataRequestUnsafe(): void;
    get allowMultiselection(): boolean;
    set allowMultiselection(value: boolean);
    forceUpdateInteractivity(): void;
    clearSelection(): void;
    protected _clearSelectionUnsafe(): void;
    protected _clearSelectionBase(): void;
    performClearSelection(): void;
    selectTuple(tuple: itemDataTupleValues, state: any): void;
    protected selectTupleUnsafe(tuple: itemDataTupleValues, state: any): void;
    setSelection(values: any): void;
    protected _setSelectionUnsafe(values: any): void;
    private setSelectionBase;
    protected _applySelection(): void;
    protected _applySelectionUnsafe(): void;
    protected _isEncodeHtml(): any;
    protected _isSupportDataAwareExport(): any;
    protected _isLocked(): boolean;
    protected _lock(): void;
    protected _unlock(): void;
    protected _getCustomSelectionMode(): any;
    protected _setCustomSelectionMode(value: any): void;
    protected _getCustomHoverEnabled(): boolean;
    protected _setCustomHoverEnabled(value: any): void;
    private _getCustomTargetAxes;
    private _setCustomTargetAxes;
    protected _getTargetAxes(): ItemDataAxisName[];
    getSelectedTuples(): itemDataTupleValues[];
    updateItem(options: any): void;
    forceUpdateItem(): void;
    private _changeTuple;
    private _onSelectionChanged;
    protected _mustSelectingFired(values: any): boolean;
    private _patchTroughDrillDownValues;
    private _deductDrillDownValues;
    private _getSelectionCallbackType;
    protected _selectTuples(tuplesToSelect: itemDataTupleValues[], unaffectedTuples: itemDataTupleValues[], isSelect: boolean): void;
    protected _selectTuplesCore(tuples: itemDataTupleValues[], updateTupleDelegate: (tuple: itemDataTupleValues) => itemDataTupleValues, state: any): void;
    protected _renderContent(element: HTMLElement, changeExisting: boolean, afterRenderCallback?: any): boolean;
    protected renderContentUnsafe($element: HTMLElement, changeExisting: boolean, afterRenderCallback?: any): boolean;
    renderPartialContent(): void;
    protected renderPartialContentUnsafe(): void;
    updateContentState(): void;
    protected updateContentStateUnsafe(): void;
    getInfo(): ViewerItemInfo;
    protected getInfoUnsafe(): ViewerItemInfo;
    private getInfoBase;
    getName(): string;
    getCaption(): any;
    hasCaption(options?: ViewerItemOptions): any;
    hasParentContainer(): boolean;
    protected _isPaneEmpty(): boolean;
    _isTransparentBackground(): boolean;
    render(container?: HTMLElement): void;
    dispose(): void;
    updateContent(newOptions: ViewerItemOptions): void;
    updateClientState(clientState: MapClientState | GridClientState): void;
    _removeShildElement(): void;
    showLoadingPanel(): void;
    hideLoadingPanel(): void;
    _toggleLoadingPanel(loading: boolean): void;
    width(width?: any): any;
    height(height?: any): any;
    setSize(width: number, height: number): void;
    getConstraints(includeBorders: any): ConstraintsEx;
    getOffset(): ISize;
    updateInteractivityOptions(): void;
    addContextCaptionToolbarOptions?: (options: DashboardItemCaptionToolbarOptions) => void;
    getCaptionToolbarOptions(): DashboardItemCaptionToolbarOptions;
    updateConstraints(): void;
    deferredToolbarRenderingPromise: (itemName: string, width: number, height: number) => JQueryPromise<any>;
    updateCaptionToolbar(): void;
    private _createCaptionToolbar;
    private _updateCaptionToolbarAndSize;
    createCaptionToolbar: (viewerItem: baseItem, container: HTMLElement, controlContainer: HTMLElement, parentContainerElement: HTMLElement, viewOptions: ItemCaptionToolbarViewOptions) => CaptionToolbar;
    forceCreateCaptionToolbar(): void;
    protected _itemHasOwnContent(): boolean;
    protected _updateClientStateUnsafe(clientState: MapClientState | GridClientState): void;
    protected _changeContent(updateExisting: boolean): void;
    protected _calcHeaderAndFooterHeight(): number;
    private _getReducedDataTooltip;
    private _getStaticToolbarItems;
    private _getReduceDataToolbarItem;
    private _getStateToolbarItems;
    private _getActionToolbarItems;
    private _getExportMenuModel;
    private _getParametersTitle;
    showExportDialog(exportFormat: any, itemElement?: DxElement): void;
    protected _getSpecificActionToolbarItems(): Array<ViewerToolbarItem>;
    protected _getSpecificStatePanelItems(): Array<ViewerToolbarItem>;
    protected _getMinContentHeight(): number;
    private _generateInnerBorderClasses;
    protected _generateInnerBorderClassesUnsafe(element?: HTMLElement): string[];
    private _generateOuterBorderClasses;
    _isBorderRequired(): boolean;
    _resize(): void;
    protected _resizeUnsafe(): void;
    private _resizeBase;
    updateContentSize(): void;
    protected _updateContentSizeUnsafe(): void;
    private _updateContentSizeBase;
    private _allocatePreloader;
    protected _getButtonOffset(useToolbarOffset: any): {
        left: any;
        top: any;
    };
    protected _getAnimationOptions(): {
        enabled: boolean;
        duration: number;
    };
    private _getContainerPosition;
    protected _getContainerPositionUnsafe(): {
        left: any;
        top: any;
        width: any;
        height: any;
        offsetX: number;
        offsetY: number;
    };
    private _getContainerPositionBase;
    protected _getBoundaryContainer(): HTMLElement;
    protected _getSelectedValues(): any[][];
    private _onClearSelection;
    protected _getElementInteractionValue(element: any, viewModel: any): void;
    protected _setSelectedValues(values: any): void;
    protected _raiseItemClick(element: any): void;
    protected _clickAction(tuple: itemDataTupleValues): void;
    protected _isMultiDataSupported(): boolean;
    protected _getDataPoint(element: any): DataPoint;
    protected _getWidget(): any;
    private _raiseItemWidgetCreated;
    private _raiseItemWidgetUpdating;
    private _raiseItemWidgetUpdated;
    protected _raiseItemWidgetOptionsPrepared(options: Object): void;
    protected _raiseItemHover(element: any, state?: any): void;
    private _onClearMasterFilter;
    private _onToggleSelectionMode;
    private _onDrillUp;
    private _onContentElementSelection;
    protected _onExpandValue(expandValueParams: any): void;
    protected _onClientStateUpdate(clientState: MapClientState | GridClientState): void;
    protected _onDataRequest(): void;
    private _hasDrillUpButton;
    private _hasClearMasterFilterButton;
    private _hasClearSelectionButton;
    protected _hasToggleSelectionModeButton(): boolean;
    private _isDrillUpEnabled;
    private _canPerformAction;
    protected _canPerformDrillDown(): boolean;
    protected _canPerformDrillUp(): boolean;
    protected _canSetMasterFilter(): boolean;
    protected _canSetMultipleMasterFilter(): boolean;
    protected isInteractivityActionEnabled(): boolean;
    protected _selectionMode(): "none" | "multiple";
    protected _getHtml(text: any): any;
    protected _getAxisNames(): ItemDataAxisName[];
    private _getDrillDownAxisName;
    private _getDrillDownValues;
}
export declare let cssClassNamesBaseItem: {
    item: string;
    groupItem: string;
    groupItemChild: string;
    simpleBorder: string;
    overlayContent: string;
    cardWihtoutBackground: string;
};
export interface ViewerItemOptions {
    Name: string;
    Type?: string;
    ParentContainer?: any;
    ContentType?: ContentType;
    SelectedValues?: any;
    ViewModel?: any;
    ActionModel?: any;
    CaptionViewModel?: any;
    ConditionalFormattingModel?: any;
    Parameters?: any;
    DrillDownValues?: any;
    DrillDownUniqueValues?: any;
    IsDrillDownEnabled?: any;
    AxisNames?: any;
    DimensionIds?: any;
    multiData?: any;
    encodeHtml?: any;
    allowExport?: boolean;
    showExportDialog?: (format: DashboardExportFormat, itemElement?: DxElement) => void;
    isDataReduced?: boolean;
    useNeutralFilterMode?: boolean;
    FullViewport?: IViewport;
    LimitDataState?: any;
    controlContainer?: HTMLElement;
    parentContainer?: any;
    boundaryContainer?: HTMLElement;
    ItemData?: any;
    DataSource?: any;
    DataSourceMembers?: any;
    animate?: any;
    useNativeScrolling?: any;
    overflow?: any;
    PivotExpandViewState?: any;
    GridClientFilterState?: any;
    forceUpdateViewModel?: boolean;
}
export interface DataPoint {
    getSelectionValues?(axisName?: ItemDataAxisName): Array<any>;
    getValues(axisName?: ItemDataAxisName): Array<any>;
    getMeasureIds?(): Array<string>;
    getDeltaIds?(): Array<string>;
}
export interface ViewerItemInfo {
    name: string;
    headerHeight: number;
    position: JQuery.Coordinates;
    width: number;
    height: number;
    virtualSize: any;
    scroll: any;
}
