﻿/**
* DevExpress Dashboard (_base-item.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { isNumeric } from '@devexpress/analytics-core/analytics-internal';
import * as $ from 'jquery';
import { contentType, viewerActions } from '../../data/_common';
import { defaultDataControllerFactory } from '../../data/_factory';
import { format } from '../../data/_formatter';
import { createJQueryCallbacks } from '../../data/_jquery-helpers';
import { localizationId } from '../../data/_localization-ids';
import { localizer } from '../../data/_localizer';
import { checkArrayContainsTuple, checkValuesAreEqual, encodeHtml, getAxisPointValue, moveContent, type } from '../../data/_utils';
import { zIndex } from '../../data/_z-index';
import { getLocalizationById } from '../../data/localization/_default';
import { DashboardLayoutModeHelper } from '../_dashboard-layout-mode-helper';
import { constraints, size } from '../layout/_utils.layout';
import { DashboardViewerConstants } from '../viewer/_dashboard-viewer-constants';
import { RenderHelper } from '../widgets/_render-helper';
import { _convertToExportFormat, cssClasses, Settings } from '../widgets/caption-toolbar/_caption-toolbar-css-classes';
import { ClickableFloatingCaptionToolbar } from '../widgets/caption-toolbar/_clickable-floating-toolbar';
import { HiddenCaptionToolbar } from '../widgets/caption-toolbar/_hidden-caption-toolbar';
import { HoveredFloatingCaptionToolbar } from '../widgets/caption-toolbar/_hovered-floating-toolbar';
import { HoveredDashboardCaptionToolbar } from '../widgets/caption-toolbar/_hovered-toolbar';
import { MinimizedClickableCaptionToolbar } from '../widgets/caption-toolbar/_minimized-clickable-toolbar';
import { MinimizedHoveredCaptionToolbar } from '../widgets/caption-toolbar/_minimized-hovered-toolbar';
import { StaticCaptionToolbar } from '../widgets/caption-toolbar/_static-toolbar';
import { dashboardToolbarItemNames } from '../widgets/caption-toolbar/caption-toolbar-options';
import { dashboardSelectionMode, interactivityController } from './_interactivity-controller';
import { ItemLoadingElement } from './elements/_item-loading';
export var createDefaultToolbar = (viewerItem, container, controlContainer, popupContainer, viewOptions) => {
    if (viewOptions.hiddenToolbar) {
        return new HiddenCaptionToolbar();
    }
    else if (DashboardLayoutModeHelper.isTouch) {
        if (viewOptions.hasCaption) {
            return new StaticCaptionToolbar(container, controlContainer, popupContainer, viewOptions.encodeHtml, cssClasses.caption, viewOptions.captionToolbarSeparatorRequired, false);
        }
        else {
            if (viewOptions.allowPreview) {
                return new MinimizedClickableCaptionToolbar(container, controlContainer, popupContainer, viewOptions.encodeHtml, viewOptions.isBottomFloatingToolbarPosition, viewOptions.itemHasOwnContent);
            }
            else {
                return new ClickableFloatingCaptionToolbar(container, controlContainer, popupContainer, viewOptions.encodeHtml, viewOptions.isBottomFloatingToolbarPosition, viewOptions.itemHasOwnContent);
            }
        }
    }
    else {
        if (viewOptions.hasCaption) {
            return new HoveredDashboardCaptionToolbar(container, controlContainer, popupContainer, viewOptions.encodeHtml, viewOptions.captionToolbarSeparatorRequired);
        }
        else {
            if (viewOptions.allowPreview) {
                return new MinimizedHoveredCaptionToolbar(container, controlContainer, popupContainer, viewOptions.encodeHtml, viewOptions.isBottomFloatingToolbarPosition);
            }
            else {
                return new HoveredFloatingCaptionToolbar(container, controlContainer, popupContainer, viewOptions.encodeHtml, viewOptions.isBottomFloatingToolbarPosition);
            }
        }
    }
};
export class baseItem {
    get hasWidget() {
        return this._hasWidget;
    }
    get _captionToolbarSeparatorRequired() {
        return false;
    }
    get _isBottomFloatingToolbarPosition() {
        return false;
    }
    get _allowPreview() {
        return false;
    }
    get _shouldApplySelectionOnInitialRender() {
        return true;
    }
    get dataController() { return null; }
    set dataController(dataController) { }
    constructor($container, options) {
        this._lockCount = 0;
        this._isFixedHeight = false;
        this.customHoverEnabled = false;
        this._allowMultiselection = false;
        this.dateToString = (date) => date.toJSON();
        this.selected = createJQueryCallbacks();
        this.clearMasterFilter = createJQueryCallbacks();
        this.drillUp = createJQueryCallbacks();
        this.contentElementSelection = createJQueryCallbacks();
        this.expandValue = createJQueryCallbacks();
        this.clientStateUpdate = createJQueryCallbacks();
        this.dataRequest = createJQueryCallbacks();
        this.itemClick = createJQueryCallbacks();
        this.itemHover = createJQueryCallbacks();
        this.itemSelectionChanged = createJQueryCallbacks();
        this.itemWidgetCreated = createJQueryCallbacks();
        this.itemWidgetUpdating = createJQueryCallbacks();
        this.itemWidgetUpdated = createJQueryCallbacks();
        this.itemWidgetOptionsPrepared = createJQueryCallbacks();
        this.itemCaptionToolbarUpdated = createJQueryCallbacks();
        this.constraintsUpdated = createJQueryCallbacks();
        this.customTargetAxes = [];
        this.customDefaultSelectedValues = [];
        this.itemLoadingElement = new ItemLoadingElement();
        this.visualMode = 'full';
        this._hasWidget = false;
        this.deferredToolbarRenderingPromise = undefined;
        this.createCaptionToolbar = createDefaultToolbar;
        this._initializeData(options);
        this.container = $container;
        this.controlContainer = options.controlContainer;
        this._boundaryContainer = options.boundaryContainer;
        this.interactivityController = new interactivityController($.proxy(this.getSelectedTuples, this));
        this.interactivityController.selectionChanged.add($.proxy(this._onSelectionChanged, this));
        this.customSelectionMode = dashboardSelectionMode.none;
    }
    _initializeData(newOptions) {
        if (!this.options) {
            this.options = newOptions;
        }
        else {
            this.options = {
                Name: newOptions.Name,
                Type: newOptions.Type,
                ParentContainer: newOptions.ParentContainer,
                ContentType: newOptions.ContentType,
                SelectedValues: newOptions.SelectedValues,
                ViewModel: newOptions.ViewModel || this.options.ViewModel,
                ActionModel: newOptions.ActionModel || this.options.ActionModel,
                CaptionViewModel: newOptions.CaptionViewModel || this.options.CaptionViewModel,
                ConditionalFormattingModel: newOptions.ConditionalFormattingModel || this.options.ConditionalFormattingModel,
                Parameters: newOptions.Parameters,
                DrillDownValues: newOptions.DrillDownValues,
                DrillDownUniqueValues: newOptions.DrillDownUniqueValues,
                IsDrillDownEnabled: newOptions.DrillDownUniqueValues,
                AxisNames: newOptions.AxisNames,
                DimensionIds: newOptions.DimensionIds,
                multiData: newOptions.multiData,
                encodeHtml: newOptions.encodeHtml !== undefined ? newOptions.encodeHtml : this.options.encodeHtml,
                allowExport: newOptions.allowExport !== undefined ? newOptions.allowExport : this.options.allowExport,
                showExportDialog: newOptions.showExportDialog !== undefined ? newOptions.showExportDialog : this.options.showExportDialog,
                isDataReduced: !!newOptions.ItemData ? newOptions.ItemData.Reduced : this.options.isDataReduced,
                useNeutralFilterMode: newOptions.useNeutralFilterMode !== undefined ? newOptions.useNeutralFilterMode : this.options.useNeutralFilterMode,
                FullViewport: newOptions.FullViewport,
                LimitDataState: newOptions.LimitDataState
            };
        }
        if (!this.dataController || (newOptions.ContentType != contentType.actionModel)) {
            var drillDownState = {};
            drillDownState[this._getDrillDownAxisName()] = this.options.DrillDownUniqueValues;
            this.dataController = defaultDataControllerFactory.createDataController(this.options.Type, {
                multiData: this.options.multiData,
                viewModel: this.options.ViewModel,
                cfModel: this.options.ConditionalFormattingModel,
                useNeutralFilterMode: this.options.useNeutralFilterMode,
                drillDownState: drillDownState
            });
            if (this.dataController) {
                this.dataController.update(this.options.SelectedValues, this._isEncodeHtml());
            }
        }
        this.customSelectedTuples = [];
    }
    initialDataRequest() {
        if (this.hasWidget) {
            this.initialDataRequestUnsafe();
        }
    }
    initialDataRequestUnsafe() {
    }
    get allowMultiselection() {
        return this._allowMultiselection;
    }
    set allowMultiselection(value) {
        this._allowMultiselection = value;
    }
    forceUpdateInteractivity() {
        this.updateCaptionToolbar();
        this.updateInteractivityOptions();
    }
    clearSelection() {
        if (this.hasWidget) {
            this._clearSelectionUnsafe();
        }
        else {
            this._clearSelectionBase();
        }
    }
    _clearSelectionUnsafe() {
        this._clearSelectionBase();
    }
    _clearSelectionBase() {
    }
    performClearSelection() {
        this._setSelectedValues(null);
        this.clearSelection();
    }
    selectTuple(tuple, state) {
        if (this.hasWidget) {
            this.selectTupleUnsafe(tuple, state);
        }
    }
    selectTupleUnsafe(tuple, state) {
    }
    setSelection(values) {
        if (this.hasWidget) {
            this._setSelectionUnsafe(values);
        }
        else {
            this.setSelectionBase(values);
        }
    }
    _setSelectionUnsafe(values) {
        this.setSelectionBase(values);
    }
    setSelectionBase(values) {
        this._setSelectedValues(values);
    }
    _applySelection() {
        if (this.hasWidget) {
            this._applySelectionUnsafe();
        }
    }
    _applySelectionUnsafe() {
    }
    _isEncodeHtml() {
        return (!this.options || this.options.encodeHtml == undefined) ? true : this.options.encodeHtml;
    }
    _isSupportDataAwareExport() {
        return this.options && this.options.allowExport && this.options.ViewModel && this.options.ViewModel.SupportDataAwareExport;
    }
    _isLocked() {
        if (this._lockCount < 0)
            throw new Error("Unresolved locker state (looks like 'unlock' method is called without 'lock')");
        return this._lockCount > 0;
    }
    _lock() {
        this._lockCount++;
    }
    _unlock() {
        this._lockCount--;
    }
    _getCustomSelectionMode() {
        return this.customSelectionMode;
    }
    _setCustomSelectionMode(value) {
        this.customSelectionMode = value;
    }
    _getCustomHoverEnabled() {
        return this.customHoverEnabled;
    }
    _setCustomHoverEnabled(value) {
        this.customHoverEnabled = value;
    }
    _getCustomTargetAxes() {
        return this.customTargetAxes;
    }
    _setCustomTargetAxes(value) {
        this.customTargetAxes = value;
    }
    _getTargetAxes() {
        if (!this.isInteractivityActionEnabled()) {
            return this._getCustomTargetAxes();
        }
        else {
            return this._getAxisNames();
        }
    }
    getSelectedTuples() {
        var that = this, multiData = that.options.multiData, axisNames = that._getAxisNames(), dimensionByAxis = {}, tupleValues, axisValues, valueIndex;
        if (that._canSetMasterFilter() || that._canSetMultipleMasterFilter() || that._canPerformDrillDown()) {
            var tuples = [];
            if (that.options.SelectedValues == null)
                return tuples;
            if (axisNames.length > 1) {
                axisNames.forEach(axisName => {
                    dimensionByAxis[axisName] = multiData.getAxis(axisName).getDimensions();
                });
                that.options.SelectedValues.forEach(selection => {
                    tupleValues = [];
                    valueIndex = 0;
                    axisNames.forEach(axisName => {
                        axisValues = [];
                        dimensionByAxis[axisName].forEach(() => {
                            axisValues.push(selection[valueIndex++]);
                        });
                        tupleValues.push({ axisName: axisName, value: axisValues });
                    });
                    tuples.push(tupleValues);
                });
            }
            else {
                var drillDownValues = that._getDrillDownValues();
                that.options.SelectedValues.forEach(value => {
                    tuples.push([{ axisName: axisNames[0], value: drillDownValues.concat(value) }]);
                });
            }
            return tuples;
        }
        else {
            return that.customSelectedTuples;
        }
    }
    updateItem(options) {
        var that = this;
        this._setCustomSelectionMode(options.selectionMode);
        this._setCustomTargetAxes(options.targetAxes);
        this._setCustomHoverEnabled(options.hoverEnabled);
        this.customDefaultSelectedValues = options.defaultSelectedValues;
        that.updateInteractivityOptions();
        if (!this.isInteractivityActionEnabled()) {
            var customDefaultSelectedValues = options.selectionMode == dashboardSelectionMode.single ? that.customDefaultSelectedValues.slice(0, 1) : that.customDefaultSelectedValues;
            that.interactivityController.clickAction(customDefaultSelectedValues);
        }
        this.updateCaptionToolbar();
    }
    forceUpdateItem() {
        this.forceCreateCaptionToolbar();
        this.updateContentSize();
    }
    _changeTuple(tuple) {
        var that = this, newTuple = [];
        tuple.forEach(axisValue => {
            var axisName = axisValue.axisName, value = axisValue.value;
            newTuple.push({ axisName: axisName, value: (axisName == that._getDrillDownAxisName()) ? value.slice(-1) : value });
        });
        return newTuple;
    }
    _onSelectionChanged(tuples) {
        var that = this, currentSelectedValues = this.options.SelectedValues, singleSelection = !!currentSelectedValues && currentSelectedValues.length === 1 ? currentSelectedValues[0] : null, newSelection = [], tupleValues, selectedTuples = this.getSelectedTuples(), actionName;
        this._selectTuples(tuples, selectedTuples, true);
        this._selectTuples(selectedTuples, tuples, false);
        if (this.isInteractivityActionEnabled()) {
            tuples.forEach(tuple => {
                tupleValues = [];
                that._getAxisNames().forEach(axisName => {
                    tupleValues.push.apply(tupleValues, getAxisPointValue(tuple, axisName).slice());
                });
                newSelection.push(tupleValues);
            });
            newSelection = this._deductDrillDownValues(newSelection);
            this._setSelectedValues(newSelection);
            actionName = this._getSelectionCallbackType(newSelection.length === 1 && singleSelection && checkValuesAreEqual(newSelection[0], singleSelection));
            if (actionName === viewerActions.drillDown) {
                newSelection = newSelection[0];
            }
            if (this._mustSelectingFired(newSelection)) {
                this.selected.fire(this.getName(), actionName, newSelection);
            }
            else {
                this._onClearMasterFilter();
            }
        }
        else {
            this.customSelectedTuples = [];
            tuples.forEach(tuple => {
                that.customSelectedTuples.push(tuple);
            });
        }
        if (this.customSelectionMode == dashboardSelectionMode.multiple) {
            this.updateCaptionToolbar();
        }
        if (this.itemSelectionChanged) {
            this.itemSelectionChanged.fire(this.getName(), tuples);
        }
    }
    _mustSelectingFired(values) {
        return values.length > 0;
    }
    _patchTroughDrillDownValues(values) {
        var drillDownValues = this._getDrillDownValues(), filterValues = [];
        if (values) {
            values.forEach(value => {
                filterValues.push(drillDownValues.concat(value));
            });
        }
        return filterValues;
    }
    _deductDrillDownValues(values) {
        var drillDownValues = this._getDrillDownValues(), drillDownValuesLength = drillDownValues.length, cutValue;
        values.forEach(value => {
            cutValue = value.slice(0, drillDownValuesLength);
            if (checkValuesAreEqual(cutValue, drillDownValues))
                value.splice(0, drillDownValuesLength);
        });
        return values;
    }
    _getSelectionCallbackType(performDrillDown) {
        var actionName = undefined;
        if (this._canSetMultipleMasterFilter() && this.allowMultiselection) {
            actionName = viewerActions.setMultipleValuesMasterFilter;
        }
        else if (this._canSetMasterFilter()) {
            if (performDrillDown) {
                if (this._canPerformDrillDown())
                    actionName = viewerActions.drillDown;
            }
            else {
                actionName = viewerActions.setMasterFilter;
            }
        }
        else if (this._canPerformDrillDown()) {
            actionName = viewerActions.drillDown;
        }
        return actionName;
    }
    _selectTuples(tuplesToSelect, unaffectedTuples, isSelect) {
        var that = this;
        var updateTuple = function (tuple) {
            if (checkArrayContainsTuple(unaffectedTuples, tuple) == undefined) {
                return that._hasDrillUpButton() && !that._isMultiDataSupported() ? that._changeTuple(tuple) : tuple;
            }
            return undefined;
        };
        this._selectTuplesCore(tuplesToSelect, updateTuple, isSelect);
    }
    _selectTuplesCore(tuples, updateTupleDelegate, state) {
        var that = this;
        $.each(tuples, function (index, tuple) {
            var updatedTuple = updateTupleDelegate(tuple);
            if (!!updatedTuple) {
                that.selectTuple(updatedTuple, state);
            }
        });
    }
    _renderContent(element, changeExisting, afterRenderCallback) {
        if (this.visualMode !== 'caption') {
            this._hasWidget = true;
            return this.renderContentUnsafe(element, changeExisting, afterRenderCallback);
        }
        return false;
    }
    renderContentUnsafe($element, changeExisting, afterRenderCallback) {
        return false;
    }
    renderPartialContent() {
        if (this.hasWidget) {
            this.renderPartialContentUnsafe();
        }
    }
    renderPartialContentUnsafe() {
    }
    updateContentState() {
        if (this.hasWidget) {
            this.updateContentStateUnsafe();
        }
    }
    updateContentStateUnsafe() {
    }
    getInfo() {
        if (this.hasWidget) {
            return this.getInfoUnsafe();
        }
        else {
            return this.getInfoBase();
        }
    }
    getInfoUnsafe() {
        return this.getInfoBase();
    }
    getInfoBase() {
        var that = this, container = this._getBoundaryContainer();
        return {
            name: that.getName(),
            headerHeight: this.hasCaption() ? this.captionToolbar.calcHeight(this.getCaptionToolbarOptions()) : 0,
            position: $.fn.constructor(container).offset(),
            width: $.fn.constructor(container).outerWidth(),
            height: $.fn.constructor(container).outerHeight(),
            virtualSize: undefined,
            scroll: undefined
        };
    }
    getName() {
        return this.options.Name;
    }
    getCaption() {
        return this.options.CaptionViewModel && this.options.CaptionViewModel.Text ? this.options.CaptionViewModel.Text.concat(this._getParametersTitle() || '') : undefined;
    }
    hasCaption(options) {
        var opts = options || this.options;
        return opts && opts.CaptionViewModel && opts.CaptionViewModel.ShowCaption;
    }
    hasParentContainer() {
        return this.options && type.isDefined(this.options.ParentContainer);
    }
    _isPaneEmpty() {
        return this.hasParentContainer();
    }
    _isTransparentBackground() {
        return false;
    }
    render(container) {
        this.container = this.container || container;
        this.container.setAttribute('data-layout-item-name', this.getName());
        this.contentRoot = document.createElement('div');
        moveContent(this.container, this.contentRoot, true);
        if (!this.captionToolbar) {
            this.captionToolbar = this._createCaptionToolbar(this.container, this.controlContainer, this._getBoundaryContainer());
            this.updateCaptionToolbar();
        }
        else if (container) {
            this.forceCreateCaptionToolbar();
        }
        this.container.appendChild(this.contentRoot);
        this.updateContentSize();
        this._changeContent(false);
    }
    dispose() {
        this._removeShildElement();
        if (this.itemLoadingElement) {
            this.itemLoadingElement.dispose();
            this.itemLoadingElement = null;
        }
        this.selected && this.selected.empty();
        this.clearMasterFilter && this.clearMasterFilter.empty();
        this.drillUp && this.drillUp.empty();
        this.contentElementSelection && this.contentElementSelection.empty();
        this.expandValue && this.expandValue.empty();
        this.clientStateUpdate && this.clientStateUpdate.empty();
        this.dataRequest && this.dataRequest.empty();
        this.itemClick && this.itemClick.empty();
        this.itemHover && this.itemHover.empty();
        this.itemSelectionChanged && this.itemSelectionChanged.empty();
        this.itemWidgetCreated && this.itemWidgetCreated.empty();
        this.itemWidgetUpdating && this.itemWidgetUpdating.empty();
        this.itemWidgetUpdated && this.itemWidgetUpdated.empty();
        this.itemWidgetOptionsPrepared && this.itemWidgetOptionsPrepared.empty();
        this.itemCaptionToolbarUpdated && this.itemCaptionToolbarUpdated.empty();
        this.constraintsUpdated && this.constraintsUpdated.empty();
        this.container = null;
        this.contentRoot = null;
        this.controlContainer = null;
        this._boundaryContainer = null;
        if (this.captionToolbar) {
            this.captionToolbar.dispose();
            this.captionToolbar = null;
        }
    }
    updateContent(newOptions) {
        var isPrevShowCaption = this.hasCaption(this.options), isNewShowCaption = this.hasCaption(newOptions), showCaptionChanged = isPrevShowCaption !== isNewShowCaption;
        this._initializeData(newOptions);
        if (type.isDefined(isNewShowCaption) && showCaptionChanged) {
            this.forceCreateCaptionToolbar();
        }
        else {
            this.updateCaptionToolbar();
        }
        switch (newOptions.ContentType) {
            case contentType.partialDataSource:
            case contentType.completeDataSource:
                if (newOptions.DataSource || (newOptions.ItemData && newOptions.ItemData.DataStorageDTO)) {
                    if (newOptions.forceUpdateViewModel)
                        this._changeContent(true);
                    else
                        this.renderPartialContent();
                }
                break;
            case contentType.actionModel:
                if (!this.isInteractivityActionEnabled()) {
                    this.updateContentState();
                }
                break;
            default:
                this._changeContent(true);
                break;
        }
    }
    updateClientState(clientState) {
        if (this.hasWidget && this.options.ContentType !== contentType.partialDataSource && this.options.ContentType !== contentType.completeDataSource)
            this._updateClientStateUnsafe(clientState);
    }
    _removeShildElement() {
        let shieldingElement = this.shieldingElement;
        if (shieldingElement) {
            shieldingElement.parentNode.removeChild(shieldingElement);
            delete this.shieldingElement;
        }
    }
    showLoadingPanel() {
        this._toggleLoadingPanel(true);
    }
    hideLoadingPanel() {
        this._toggleLoadingPanel(false);
    }
    _toggleLoadingPanel(loading) {
        if (!this.container)
            return;
        const container = this._getBoundaryContainer();
        if (this.visualMode !== 'caption') {
            if (!!loading) {
                this.itemLoadingElement.show(container);
            }
            else {
                this.itemLoadingElement.hide();
            }
        }
        if (!loading) {
            this._removeShildElement();
        }
        else if (!this.shieldingElement) {
            const position = this._getContainerPosition();
            const shieldingElement = document.createElement('div');
            shieldingElement.classList.add('dx-dashboard-item-shield');
            shieldingElement.style.left = position.left.toString() + 'px';
            shieldingElement.style.top = position.top.toString() + 'px';
            shieldingElement.style.width = position.width.toString() + 'px';
            shieldingElement.style.height = position.height.toString() + 'px';
            shieldingElement.style.zIndex = zIndex.dashboardItemShield.toString();
            this.shieldingElement = shieldingElement;
            $.fn.constructor(container).prepend(shieldingElement);
        }
    }
    width(width) {
        var that = this;
        if (isNumeric(width)) {
            that.setSize(width, undefined);
        }
        else {
            return $.fn.constructor(that.container).outerWidth();
        }
    }
    height(height) {
        var that = this;
        if (isNumeric(height)) {
            that.setSize(undefined, height);
        }
        else {
            return $.fn.constructor(that.container).outerHeight();
        }
    }
    setSize(width, height) {
        var that = this, oldSize = { width: that.width(), height: that.height() }, newSize = { width: width, height: height };
        if (width) {
            $.fn.constructor(that.container).outerWidth(width);
        }
        if (height) {
            $.fn.constructor(that.container).outerHeight(height);
        }
        that._resize();
    }
    getConstraints(includeBorders) {
        var borderSize = includeBorders ? RenderHelper.getBorderSizeByClasses(this._generateOuterBorderClasses().concat(this._generateInnerBorderClasses())) : { width: 0, height: 0 }, headerHeight = this._calcHeaderAndFooterHeight(), contentMinHeight = this._getMinContentHeight(), height = borderSize.height + headerHeight + contentMinHeight;
        return constraints(size(DashboardViewerConstants.minPaneWidth + borderSize.width, height), size(Number.MAX_VALUE, this._isFixedHeight ? height : Number.MAX_VALUE));
    }
    getOffset() {
        return {
            width: 0,
            height: 0
        };
    }
    updateInteractivityOptions() {
        var that = this, selectionMode = dashboardSelectionMode.none;
        if (!that.isInteractivityActionEnabled()) {
            that.updateContentState();
            selectionMode = this.customSelectionMode;
            if (selectionMode == dashboardSelectionMode.multiple && !this.allowMultiselection)
                selectionMode = dashboardSelectionMode.single;
        }
        else {
            if (this._canSetMultipleMasterFilter() && this.allowMultiselection) {
                selectionMode = dashboardSelectionMode.multiple;
            }
            else {
                if (that.isInteractivityActionEnabled()) {
                    selectionMode = dashboardSelectionMode.single;
                }
            }
        }
        that.interactivityController.setOptions(selectionMode);
    }
    getCaptionToolbarOptions() {
        let actionAtems = this._getActionToolbarItems();
        let staticItems = this._getStaticToolbarItems();
        let stateItems = this._getStateToolbarItems();
        var toolbarOptions = {
            staticItems: staticItems,
            actionItems: actionAtems,
            stateItems: stateItems,
            navigationItems: []
        };
        this.addContextCaptionToolbarOptions && this.addContextCaptionToolbarOptions(toolbarOptions);
        this.itemCaptionToolbarUpdated.fire(this.getName(), toolbarOptions);
        return toolbarOptions;
    }
    updateConstraints() {
        this.constraintsUpdated.fire(this.getName());
    }
    updateCaptionToolbar() {
        let options = this.getCaptionToolbarOptions();
        if (this.deferredToolbarRenderingPromise) {
            this.deferredToolbarRenderingPromise(this.getName(), this.captionToolbar.calcMinWidth(options), this.captionToolbar.calcHeight(options)).done(() => {
                this._updateCaptionToolbarAndSize(options);
            });
        }
        else {
            this._updateCaptionToolbarAndSize(options);
        }
    }
    _createCaptionToolbar(container, controlContainer, popupContainer) {
        return this.createCaptionToolbar(this, container, controlContainer, popupContainer, {
            encodeHtml: this._isEncodeHtml(),
            hasCaption: this.hasCaption(),
            isBottomFloatingToolbarPosition: this._isBottomFloatingToolbarPosition,
            captionToolbarSeparatorRequired: this._captionToolbarSeparatorRequired,
            allowPreview: this._allowPreview,
            hiddenToolbar: this.visualMode === 'content',
            itemHasOwnContent: this._itemHasOwnContent()
        });
    }
    _updateCaptionToolbarAndSize(options) {
        let heightChanged = this.captionToolbar.update(options);
        if (heightChanged) {
            this.updateContentSize();
        }
    }
    forceCreateCaptionToolbar() {
        this.captionToolbar.dispose();
        this.captionToolbar = this._createCaptionToolbar(this.container, this.controlContainer, this._getBoundaryContainer());
        this.updateCaptionToolbar();
    }
    _itemHasOwnContent() {
        return true;
    }
    _updateClientStateUnsafe(clientState) {
    }
    _changeContent(updateExisting) {
        if (this.visualMode === 'caption')
            return;
        let afterRenderCallback = () => {
            if (updateExisting) {
                this._raiseItemWidgetUpdated();
            }
            else {
                this._raiseItemWidgetCreated();
            }
            if (updateExisting || this._shouldApplySelectionOnInitialRender)
                this._applySelection();
        };
        if (updateExisting) {
            this._raiseItemWidgetUpdating();
        }
        if (!this._renderContent(this.contentRoot, updateExisting, afterRenderCallback)) {
            afterRenderCallback();
        }
    }
    _calcHeaderAndFooterHeight() {
        var headerAndFooterHeight = 0;
        let toolbar = this._createCaptionToolbar(undefined, undefined, undefined);
        headerAndFooterHeight += toolbar.calcHeight(this.getCaptionToolbarOptions());
        toolbar.dispose();
        return headerAndFooterHeight;
    }
    _getReducedDataTooltip() {
        if (this.options.LimitDataState) {
            if (this.options.LimitDataState.isReduced)
                return getLocalizationById('DashboardWebStringId.LimitVisibleData.DisplayAllDataTooltip');
            return getLocalizationById('DashboardWebStringId.LimitVisibleData.IncreasePerformanceTooltip');
        }
        return undefined;
    }
    _getStaticToolbarItems() {
        let items = [];
        if (this.hasCaption()) {
            let caption = this.getCaption();
            if (caption) {
                items.push({
                    name: dashboardToolbarItemNames.itemCaption,
                    type: 'text',
                    text: caption
                });
            }
            if (this.allowLimitDataCallback && this.options.LimitDataState.isReduceMode)
                items.push(this._getReduceDataToolbarItem());
        }
        return items;
    }
    _getReduceDataToolbarItem() {
        return {
            name: dashboardToolbarItemNames.limitVisibleData,
            checked: this.options.LimitDataState.isReduced,
            click: (element) => { this.allowLimitDataCallback(); },
            icon: cssClasses.iconLimitVisibleData,
            type: 'button',
            tooltip: {
                className: cssClasses.tooltipLimitVisibleData,
                template: () => {
                    let div = document.createElement('div');
                    div.innerText = this._getReducedDataTooltip();
                    return div;
                }
            }
        };
    }
    _getStateToolbarItems() {
        let items = this._getSpecificStatePanelItems();
        let actionModel = this.options.ActionModel;
        if (this._hasClearMasterFilterButton()) {
            items.push({
                name: dashboardToolbarItemNames.clearMasterFilter,
                click: (element) => {
                    element = element?.parentElement;
                    while (element && ((!element.hasAttribute('tabindex') || element.getAttribute('tabindex') === '-1') || !element.offsetParent)) {
                        element = element.parentElement;
                    }
                    this._onClearMasterFilter();
                    element && element.focus();
                },
                icon: cssClasses.iconClearMasterFilter,
                type: 'button',
                hint: localizer.getString(localizationId.buttonNames.ClearMasterFilter)
            });
        }
        if (this.options.useNeutralFilterMode && actionModel && actionModel.ClearMasterFilterButtonState === 'Disabled') {
            items.push({
                name: dashboardToolbarItemNames.clearMasterFilter,
                icon: cssClasses.iconClearMasterFilter,
                type: 'button',
                disabled: true,
                hint: localizer.getString(localizationId.buttonNames.ClearMasterFilter)
            });
        }
        if (this._hasClearSelectionButton())
            items.push({
                name: dashboardToolbarItemNames.clearSelection,
                click: (element) => { this._onClearSelection(); },
                icon: cssClasses.iconClearSelection,
                type: 'button',
                hint: localizer.getString(localizationId.buttonNames.ClearSelection)
            });
        if (this._hasDrillUpButton() && this._isDrillUpEnabled())
            items.push({
                name: dashboardToolbarItemNames.drillUp,
                click: (element) => { this._onDrillUp(); },
                icon: cssClasses.iconDrillUp,
                type: 'button',
                hint: localizer.getString(localizationId.buttonNames.DrillUp)
            });
        if (!this.hasCaption() && this.allowLimitDataCallback && this.options.LimitDataState.isReduceMode)
            items.push(this._getReduceDataToolbarItem());
        return items;
    }
    _getActionToolbarItems() {
        let items = this._getSpecificActionToolbarItems(), contentDescription = this.options.ViewModel ? this.options.ViewModel.ContentDescription : undefined;
        if (this.options.allowExport)
            items.push({
                name: dashboardToolbarItemNames.exportMenu,
                menu: this._getExportMenuModel(),
                icon: cssClasses.iconItemExport,
                type: 'menu',
                hint: localizer.getString(localizationId.buttonNames.ExportTo)
            });
        if (contentDescription && contentDescription.ElementSelectionEnabled)
            items.push({
                name: dashboardToolbarItemNames.values,
                menu: {
                    items: contentDescription.ElementNames,
                    selectedItems: [contentDescription.ElementNames[contentDescription.SelectedElementIndex]],
                    selectionMode: 'single',
                    itemClick: (itemData, itemElement, index) => { this._onContentElementSelection(index); },
                    type: 'list'
                },
                icon: cssClasses.iconContentSelection,
                type: 'menu',
                hint: localizer.getString(localizationId.buttonNames.ElementSelection)
            });
        if (this._hasToggleSelectionModeButton())
            items.push({
                name: dashboardToolbarItemNames.multiselection,
                checked: this.allowMultiselection,
                click: (element) => { this._onToggleSelectionMode(); },
                icon: cssClasses.iconMultiselection,
                type: 'button',
                hint: localizer.getString(localizationId.buttonNames.AllowMultiselection)
            });
        return items;
    }
    _getExportMenuModel() {
        let items = [cssClasses.iconExportToPDF];
        if (Settings.allowExportToImage)
            items.push(cssClasses.iconExportToImage);
        if (this._isSupportDataAwareExport()) {
            items.push(cssClasses.iconExportToExcel);
        }
        return {
            title: localizer.getString(localizationId.buttonNames.ExportTo),
            items: items,
            itemClick: (itemData, itemElement, index) => { this.showExportDialog(_convertToExportFormat(itemData), itemElement); },
            type: 'icons'
        };
    }
    _getParametersTitle() {
        if (!this.hasCaption())
            return undefined;
        let parameters = [], separator = ' - ', drillDownValues = this.options.DrillDownValues, contentDescription = this.options.ViewModel.ContentDescription;
        if (contentDescription && contentDescription.ElementSelectionEnabled) {
            parameters.push(contentDescription.ElementNames[contentDescription.SelectedElementIndex || 0]);
        }
        if (drillDownValues && drillDownValues.length > 0) {
            parameters = parameters.concat(drillDownValues.map(drillDownValue => format(drillDownValue.Value, drillDownValue.Format)));
        }
        return parameters.length > 0 ? separator + parameters.join(separator) : undefined;
    }
    showExportDialog(exportFormat, itemElement) {
        this.options.showExportDialog(exportFormat, itemElement);
    }
    _getSpecificActionToolbarItems() {
        return [];
    }
    _getSpecificStatePanelItems() {
        return [];
    }
    _getMinContentHeight() {
        return DashboardViewerConstants.minPaneHeight;
    }
    _generateInnerBorderClasses(element) {
        if (this.visualMode === 'caption') {
            return [];
        }
        else {
            return this._generateInnerBorderClassesUnsafe(element);
        }
    }
    _generateInnerBorderClassesUnsafe(element) {
        var classes = [cssClassNamesBaseItem.item];
        if (this._isBorderRequired()) {
            classes.push(cssClassNamesBaseItem.simpleBorder);
        }
        if (element) {
            element.classList.add(cssClassNamesBaseItem.item);
            element.setAttribute('tabindex', '0');
            if (this._isBorderRequired()) {
                element.classList.add(cssClassNamesBaseItem.simpleBorder);
            }
            else {
                element.classList.remove(cssClassNamesBaseItem.simpleBorder);
            }
        }
        return classes;
    }
    _generateOuterBorderClasses(element) {
        if (this.visualMode === 'caption')
            return;
        if (element) {
            if (this.hasParentContainer() && this.visualMode === 'full') {
                element.classList.add(cssClassNamesBaseItem.groupItemChild);
            }
            else {
                element.classList.remove(cssClassNamesBaseItem.groupItemChild);
            }
        }
        return this.hasParentContainer() && this.visualMode === 'full' ? [cssClassNamesBaseItem.groupItemChild] : [];
    }
    _isBorderRequired() {
        return this._isPaneEmpty() && this.visualMode === 'full';
    }
    _resize() {
        if (this.hasWidget) {
            this._resizeUnsafe();
        }
        else {
            this._resizeBase();
        }
    }
    _resizeUnsafe() {
        this._resizeBase();
    }
    _resizeBase() {
        this.updateContentSize();
        this._allocatePreloader();
        this.captionToolbar.onResize();
    }
    updateContentSize() {
        if (this.hasWidget) {
            this._updateContentSizeUnsafe();
        }
        else {
            this._updateContentSizeBase();
        }
    }
    _updateContentSizeUnsafe() {
        this._updateContentSizeBase();
    }
    _updateContentSizeBase() {
        var that = this;
        that._generateOuterBorderClasses(that.container);
        that._generateInnerBorderClasses(that.contentRoot);
        var contentHeight = Math.floor($.fn.constructor(that.container).height() - that._calcHeaderAndFooterHeight());
        $.fn.constructor(that.contentRoot).outerHeight(contentHeight);
    }
    _allocatePreloader() {
        let shieldingElement = this.container.querySelector('.dx-dashboard-item-shield'), position = this._getContainerPosition();
        this.itemLoadingElement.resize();
        if (shieldingElement) {
            shieldingElement.style.left = position.left.toString() + 'px';
            shieldingElement.style.top = position.top.toString() + 'px';
            shieldingElement.style.width = position.width.toString() + 'px';
            shieldingElement.style.height = position.height.toString() + 'px';
        }
    }
    _getButtonOffset(useToolbarOffset) {
        var defaultButtonSize = 28;
        var hoverDiv = useToolbarOffset ? this.container.querySelector('.' + cssClassNamesBaseItem.overlayContent) : undefined;
        return {
            left: !!hoverDiv ? $.fn.constructor(hoverDiv).outerWidth(true) : defaultButtonSize,
            top: !!hoverDiv ? $.fn.constructor(hoverDiv).outerHeight(true) : defaultButtonSize
        };
    }
    _getAnimationOptions() {
        return {
            enabled: !!this.options.animate,
            duration: 300
        };
    }
    _getContainerPosition() {
        if (this.hasWidget) {
            return this._getContainerPositionUnsafe();
        }
        else {
            return this._getContainerPositionBase();
        }
    }
    _getContainerPositionUnsafe() {
        return this._getContainerPositionBase();
    }
    _getContainerPositionBase() {
        var container = this._getBoundaryContainer(), position = $.fn.constructor(container).position(), width = $.fn.constructor(container).outerWidth(), height = $.fn.constructor(container).outerHeight(), marginX = $.fn.constructor(container).css('margin-left'), marginY = $.fn.constructor(container).css('margin-top'), border = this._isPaneEmpty() ? 0 : 1, parseMargin = function (margin) { return margin == 'auto' ? 0 : parseInt(margin); };
        return {
            left: position.left,
            top: position.top,
            width: width,
            height: height,
            offsetX: width + parseMargin(marginX) - border,
            offsetY: parseMargin(marginY) - border
        };
    }
    _getBoundaryContainer() {
        return this.visualMode === 'caption' ? this._boundaryContainer : this.container;
    }
    _getSelectedValues() {
        var selectedValues = this.options.SelectedValues;
        return this._isMultiDataSupported() ? this._patchTroughDrillDownValues(selectedValues) : selectedValues;
    }
    _onClearSelection() {
        this.clearSelection();
        this.customSelectedTuples = [];
        if (this.customSelectionMode == dashboardSelectionMode.multiple)
            this.updateCaptionToolbar();
        this.itemSelectionChanged.fire(this.getName(), []);
    }
    _getElementInteractionValue(element, viewModel) {
    }
    _setSelectedValues(values) {
        this.options.SelectedValues = values;
    }
    _raiseItemClick(element) {
        var that = this, tuple = [], dataPoint = that._getDataPoint(element), drillDownValues = that._getDrillDownValues(), targetAxes = that._getTargetAxes(), drillDownAxis = drillDownValues && targetAxes.length == 1 ? targetAxes[0] : undefined;
        targetAxes.forEach(axisName => {
            var values;
            if (dataPoint.getSelectionValues) {
                values = dataPoint.getSelectionValues(axisName);
            }
            else {
                values = dataPoint.getValues(axisName);
            }
            if (values.length > 0) {
                if (drillDownAxis && axisName === drillDownAxis && !that._isMultiDataSupported()) {
                    values = drillDownValues.concat(values);
                }
                tuple.push({ axisName: axisName, value: values });
            }
        });
        if ((targetAxes.length != 0) && (targetAxes.length == tuple.length)) {
            that._clickAction(tuple);
        }
        if (that.itemClick) {
            that.itemClick.fire(that.getName(), dataPoint);
        }
    }
    _clickAction(tuple) {
        this.interactivityController.clickAction([tuple]);
    }
    _isMultiDataSupported() {
        return false;
    }
    _getDataPoint(element) {
        return null;
    }
    _getWidget() {
        return null;
    }
    _raiseItemWidgetCreated() {
        var widget = this._getWidget();
        if (widget) {
            this.itemWidgetCreated.fire(this.getName(), widget);
        }
    }
    _raiseItemWidgetUpdating() {
        var widget = this._getWidget();
        if (widget) {
            this.itemWidgetUpdating.fire(this.getName(), widget);
        }
    }
    _raiseItemWidgetUpdated() {
        var widget = this._getWidget();
        if (widget) {
            this.itemWidgetUpdated.fire(this.getName(), widget);
        }
    }
    _raiseItemWidgetOptionsPrepared(options) {
        if (options) {
            this.itemWidgetOptionsPrepared.fire(this.getName(), options);
        }
    }
    _raiseItemHover(element, state) {
        if (this.itemHover) {
            var dataPoint = this._getDataPoint(element);
            this.itemHover.fire(this.getName(), dataPoint, state);
        }
    }
    _onClearMasterFilter() {
        var name = this.getName();
        this._setSelectedValues(null);
        if (!this.options || !this.options.useNeutralFilterMode) {
            this.clearSelection();
        }
        this.clearMasterFilter.fire(name);
    }
    _onToggleSelectionMode() {
        this._allowMultiselection = !this._allowMultiselection;
        this.allowMultiselectionChanged && this.allowMultiselectionChanged(this._allowMultiselection);
        this.updateCaptionToolbar();
        this.updateInteractivityOptions();
    }
    _onDrillUp() {
        this.drillUp.fire(this.getName(), !!this._getSelectedValues());
    }
    _onContentElementSelection(index) {
        this.contentElementSelection.fire(this.getName(), {
            index: index,
            caption: this.options.ViewModel.ContentDescription.ElementNames[index]
        });
    }
    _onExpandValue(expandValueParams) {
        this.expandValue.fire(this.getName(), expandValueParams);
    }
    _onClientStateUpdate(clientState) {
        this.clientStateUpdate.fire(this.getName(), clientState);
    }
    _onDataRequest() {
        this.dataRequest.fire(this.getName());
    }
    _hasDrillUpButton() {
        var actionModel = this.options.ActionModel;
        return actionModel && actionModel.DrillUpButtonState && actionModel.DrillUpButtonState !== 'Hidden';
    }
    _hasClearMasterFilterButton() {
        var actionModel = this.options.ActionModel;
        return actionModel && actionModel.ClearMasterFilterButtonState && actionModel.ClearMasterFilterButtonState === 'Enabled';
    }
    _hasClearSelectionButton() {
        return !this.isInteractivityActionEnabled() && this.customSelectionMode == dashboardSelectionMode.multiple && this.customSelectedTuples.length > 0;
    }
    _hasToggleSelectionModeButton() {
        return this._canSetMultipleMasterFilter() || (!this.isInteractivityActionEnabled() && this.customSelectionMode == dashboardSelectionMode.multiple);
    }
    _isDrillUpEnabled() {
        var actionModel = this.options.ActionModel;
        return actionModel && actionModel.DrillUpButtonState && actionModel.DrillUpButtonState === 'Enabled';
    }
    _canPerformAction(action) {
        var actionModel = this.options.ActionModel;
        return actionModel && actionModel.Actions && actionModel.Actions.indexOf(action) !== -1;
    }
    _canPerformDrillDown() {
        return this._canPerformAction(viewerActions.drillDown);
    }
    _canPerformDrillUp() {
        return this._canPerformAction(viewerActions.drillUp);
    }
    _canSetMasterFilter() {
        return this._canPerformAction(viewerActions.setMasterFilter);
    }
    _canSetMultipleMasterFilter() {
        return this._canPerformAction(viewerActions.setMultipleValuesMasterFilter);
    }
    isInteractivityActionEnabled() {
        return this._canSetMasterFilter() || this._canSetMultipleMasterFilter() || this._canPerformDrillDown();
    }
    _selectionMode() {
        return this.isInteractivityActionEnabled() ? 'multiple' : 'none';
    }
    _getHtml(text) {
        return this._isEncodeHtml() ? encodeHtml(text) : text;
    }
    _getAxisNames() {
        return this.options.AxisNames || [];
    }
    _getDrillDownAxisName() {
        return this._getAxisNames().length > 0 ? this._getAxisNames()[0] : undefined;
    }
    _getDrillDownValues() {
        var drillDownValues = this.options.DrillDownUniqueValues;
        return drillDownValues != null ? drillDownValues : [];
    }
}
export let cssClassNamesBaseItem = {
    item: 'dx-dashboard-item',
    groupItem: 'dx-dashboard-group-item',
    groupItemChild: 'dx-dashboard-group-item-child',
    simpleBorder: 'dx-dashboard-simple-border',
    overlayContent: 'dx-overlay-content',
    cardWihtoutBackground: 'dx-dashboard-card-without-background'
};
