﻿/**
* DevExpress Dashboard (_item-loading.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import dxOverlay from 'devextreme/ui/overlay/ui.overlay';
import 'devextreme/ui/toast';
import * as $ from 'jquery';
import { $unwrap } from '../../../data/_jquery-helpers';
import { zIndex } from '../../../data/_z-index';
import { getLocalizationById } from '../../../data/localization/_default';
export class ItemLoadingElement {
    show(container) {
        if (!this._overlay) {
            this._overlayDiv = document.createElement('div');
            const contentTemplate = document.createElement('div');
            contentTemplate.classList.add('dx-dashboard-item-loading-panel');
            const h1 = document.createElement('h1');
            h1.innerText = getLocalizationById('DashboardStringId.MessageLoading');
            contentTemplate.appendChild(h1);
            this._overlay = new dxOverlay(this._overlayDiv, {
                shading: true,
                animation: {
                    show: { type: 'fade', duration: 150, from: 0, to: 1, delay: 150 },
                    hide: { type: 'fade', duration: 150, to: 0, delay: 150 }
                },
                width: 'auto',
                height: 'auto',
                target: container,
                container: container,
                contentTemplate: contentTemplate,
                hideOnOutsideClick: false,
                propagateOutsideClick: true,
                position: {
                    boundary: container,
                    my: 'bottom right',
                    at: 'bottom right',
                    offset: '-10 -10',
                    of: container
                },
                onShowing: (e) => {
                    let wrapper = $unwrap(e.component._$wrapper);
                    wrapper.style.zIndex = (zIndex.dashboardItemShield - 2).toString();
                    let overlayContent = wrapper.querySelector('.dx-overlay-content');
                    if (overlayContent) {
                        overlayContent.style.zIndex = (zIndex.dashboardItemShield - 1).toString();
                    }
                },
                wrapperAttr: {
                    class: 'dx-dashboard-loading-indicator'
                }
            });
            this._overlayDiv.style.zIndex = zIndex.dashboardItemShield.toString();
            $.fn.constructor(container).prepend(this._overlayDiv);
        }
        this._overlay.show();
    }
    hide() {
        this._overlay?.hide();
    }
    resize() {
        this._overlay?.repaint();
    }
    dispose() {
        this.hide();
        this._overlay?.dispose();
        this._overlay = null;
        if (this._overlayDiv && this._overlayDiv.parentNode) {
            this._overlayDiv.parentNode.removeChild(this._overlayDiv);
            this._overlayDiv = null;
        }
    }
}
