﻿/**
* DevExpress Dashboard (_base-element.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import ArrayStore from 'devextreme/data/array_store';
import DataSource from 'devextreme/data/data_source';
import { KEY_EXPR } from '../../../data/data-controllers/_filter-element-data-controller';
import { baseItem } from '../_base-item';
import { dashboardSelectionMode } from '../_interactivity-controller';
var PAGE_SIZE = 100;
export class filterElementBaseItem extends baseItem {
    constructor(container, options) {
        super(container, options);
    }
    dispose() {
        super.dispose();
        this.widget && this.widget.dispose();
    }
    _setSelectionUnsafe(values) {
        super._setSelectionUnsafe(values);
        this.filterDataController.update(values, this._isEncodeHtml(), true);
    }
    renderContentUnsafe(element, changeExisting, afterRenderCallback) {
        var opts = this._getOptions(true);
        this._raiseItemWidgetOptionsPrepared(opts);
        this._lock();
        try {
            if (changeExisting && this.widget && this.widget.NAME === this._getWidgetName()) {
                this.widget.option(opts);
            }
            else {
                var div = this._createWidgetDiv();
                element.innerHTML = '';
                element.appendChild(div);
                if (this.widget && this.widget.dispose)
                    this.widget.dispose();
                this.widget = this._createWidget(div, opts);
            }
        }
        finally {
            this._unlock();
        }
        return false;
    }
    _createWidgetDiv() {
        return document.createElement('div');
    }
    _initializeData(newOptions) {
        super._initializeData(newOptions);
        if (this.options) {
            this.options.allowExport = false;
        }
    }
    get allowMultiselection() {
        return true;
    }
    set allowMultiselection(value) {
    }
    get isMultiSelectable() {
        return this.dataController ? this.dataController.isMultiselectable() : false;
    }
    _isPaneEmpty() {
        return super._isPaneEmpty() || !this.hasCaption();
    }
    updateInteractivityOptions() {
        this.interactivityController.setOptions(this.isMultiSelectable ? dashboardSelectionMode.multiple : dashboardSelectionMode.single);
        this.interactivityController._allowSelectTuple = function (tuple) { return true; };
    }
    getDataSource() {
        return new DataSource({
            paginate: true,
            pageSize: PAGE_SIZE,
            requireTotalCount: true,
            store: new ArrayStore({
                data: this.filterDataController.dataSource,
                key: KEY_EXPR
            }),
            searchOperation: 'contains',
            searchExpr: this._getDisplayExpr()
        });
    }
    get _enableSearch() {
        return !!this.options && !!this.options.ViewModel && this.options.ViewModel.EnableSearch;
    }
    get _isBottomFloatingToolbarPosition() {
        return true;
    }
    _isBorderRequired() {
        return false;
    }
    _getDisplayExpr() {
        return this._isEncodeHtml() ? 'text' : 'html';
    }
    _getOptions(includeActions) {
        return {};
    }
    _hasToggleSelectionModeButton() {
        return false;
    }
    _resizeUnsafe() {
        super._resizeUnsafe();
        if (this.widget.updateDimensions) {
            this.widget.updateDimensions();
        }
    }
    _raiseItemClick(elements) {
        if (this._isLocked()) {
            return;
        }
        var axisName = this._getAxisNames()[0], newSelectedValues = this.filterDataController.getInteractionValues(elements, this._getSelectedValues()), tuples = !newSelectedValues ? null : newSelectedValues.map(v => [{ axisName: axisName, value: v }]);
        this.interactivityController.clickAction(tuples, !newSelectedValues);
    }
    _mustSelectingFired(values) {
        return true;
    }
    _isUpdating(widget) {
        return !widget || widget._updateLockCount > 0;
    }
    _applySelectionUnsafe() {
        var selectedValues = this._getSelectedValues();
        if (!!selectedValues) {
            this.setSelection(selectedValues);
        }
    }
    _selectTuples(tuplesToSelect, unaffectedTuples, isSelect) {
    }
    _getWidget() {
        return this.widget;
    }
    _getWidgetName() {
        return '';
    }
    _createWidget(div, opts) {
        return null;
    }
}
