﻿/**
* DevExpress Dashboard (_date-filter-element.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { viewerActions } from '../../../data/_common';
import { DashboardFormatHelper } from '../../../data/_format-helper';
import { convertToFormat } from '../../../data/_formatter';
import { getHeight, getWidth } from '../../../data/_jquery-helpers';
import { baseItem } from '../_base-item';
import { DateFilterWidget } from './date-filter-widget';
export let cssDateFilterClassNames = {
    item: 'dx-dashboard-date-filter-item'
};
export class dateFilterElement extends baseItem {
    get _allowPreview() {
        return true;
    }
    constructor(container, options) {
        super(container, options);
    }
    _getMinContentHeight() {
        if (this.widget && this._isFixedHeight)
            return this.widget._getHeight();
        return 0;
    }
    renderContentUnsafe(element, changeExisting, afterRenderCallback) {
        super.renderContentUnsafe(element, changeExisting, afterRenderCallback);
        var isFirstInitialization = !changeExisting || !this.widget;
        if (isFirstInitialization)
            this.widget = new DateFilterWidget(element, { useNativeScrolling: this.options.useNativeScrolling, overflow: this.options.overflow }, this.controlContainer);
        var options = this.getWidgetOptions();
        this._lock();
        this.widget._update(options);
        this._unlock();
        this.widget._updateSize(getWidth(this.contentRoot), getHeight(this.contentRoot));
        this._isFixedHeight = options.arrangementMode === 'AutoHeight';
        this.timePeriodMenuSelectedIndex = this.options.ViewModel.SelectedPeriodIndex;
        this._applySelectionToWidget();
        return false;
    }
    getInfoUnsafe() {
        return {
            ...super.getInfoUnsafe(),
            selectedPeriodName: this._getCurrentPredefinedRange()
        };
    }
    _setSelectionUnsafe(values) {
        super._setSelectionUnsafe(values);
        this._applySelectionToWidget();
    }
    _clearSelectionUnsafe() {
        super._clearSelectionUnsafe();
        if (this.widget) {
            this.timePeriodMenuSelectedIndex = undefined;
            this._clearSelectedValues();
        }
    }
    _initializeData(newOptions) {
        super._initializeData(newOptions);
        if (this.options) {
            this.options.allowExport = false;
        }
    }
    _clearSelectedValues() {
        this._lock();
        this.widget._clearSelectedValues();
        this._unlock();
    }
    _applySelectionToWidget() {
        this._lock();
        if (this.timePeriodMenuSelectedIndex !== undefined) {
            this.widget._setPeriod(this.timePeriodMenuSelectedIndex);
        }
        else {
            const selectedValues = this.options.SelectedValues && this.options.SelectedValues[0] && this.options.SelectedValues[0].map(v => this._convertSelectedValues(v));
            this.widget._setSelectedValues(selectedValues);
        }
        this._unlock();
        this.updateConstraints();
    }
    getWidgetOptions() {
        var multiData = this.options.multiData;
        var filterViewModel = this.options.ViewModel;
        var getValue = (name) => {
            return multiData && multiData.getMeasureValue(name).getValue();
        };
        return {
            filterType: filterViewModel.FilterType,
            arrangementMode: filterViewModel.ArrangementMode,
            datePickerLocation: filterViewModel.DatePickerLocation,
            displayTextPattern: filterViewModel.DisplayTextPattern,
            groupInterval: filterViewModel.GroupInterval,
            format: (value) => {
                var valueToFormat = value;
                if (filterViewModel.GroupInterval === 'Year')
                    valueToFormat = this._ensureYearValue(valueToFormat);
                return DashboardFormatHelper.format(valueToFormat, convertToFormat({ DateTimeFormat: filterViewModel.DateTimeFormat }));
            },
            buttonClick: (name) => {
                if (!this._isLocked()) {
                    for (var i = 0; i < filterViewModel.DateTimePeriods.length; i++) {
                        if (name === filterViewModel.DateTimePeriods[i].Name && (name !== this._getCurrentPredefinedRange())) {
                            this.predefinedRangeChanged && this.predefinedRangeChanged(name);
                            break;
                        }
                    }
                }
            },
            buttonsInfo: filterViewModel.DateTimePeriods.map(period => period.Name),
            valueChanged: values => {
                if (!this._isLocked())
                    this.selected.fire(this.getName(), viewerActions.setMasterFilter, [values]);
            },
            minimum: getValue(filterViewModel.MinimumId),
            maximum: getValue(filterViewModel.MaximumId),
            clearAction: () => this.clearMasterFilter.fire(this.getName()),
            hasOuterMargin: !this._isPaneEmpty() || this.visualMode === 'content',
            mobileLayout: this._mobileLayout()
        };
    }
    _mobileLayout() {
        return false;
    }
    _convertSelectedValues(value) {
        return value != null && this.options.ViewModel.GroupInterval == 'Year' && typeof value === 'number' ? new Date(value, 0, 1) : value;
    }
    _getCurrentPredefinedRange() {
        return this.timePeriodMenuSelectedIndex !== undefined ? this.options.ViewModel.DateTimePeriods[this.timePeriodMenuSelectedIndex].Name : '';
    }
    _setPredefinedRange(dateTimePeriodName) {
        if (dateTimePeriodName) {
            var dateTimePeriods = this.options.ViewModel.DateTimePeriods;
            let period = dateTimePeriods.filter(dateTimePeriod => dateTimePeriod.Name === dateTimePeriodName)[0];
            if (period) {
                let index = dateTimePeriods.indexOf(period);
                this.timePeriodMenuSelectedIndex = index;
                this._applySelectionToWidget();
            }
        }
        else {
            this.timePeriodMenuSelectedIndex = undefined;
        }
    }
    _ensureYearValue(value) {
        return value instanceof Date ? value.getFullYear() : value;
    }
    _updateContentSizeUnsafe() {
        super._updateContentSizeUnsafe();
        var newContentWidth = getWidth(this.contentRoot), newContentHeight = getHeight(this.contentRoot);
        if (this.options.ViewModel.ArrangementMode === 'AutoHeight') {
            var oldMinContentHeight = this._getMinContentHeight();
            this.widget._updateSize(newContentWidth, newContentHeight);
            var newMinContentHeight = this._getMinContentHeight();
            if (oldMinContentHeight && oldMinContentHeight !== newMinContentHeight) {
                this.updateConstraints();
            }
        }
        else
            this.widget._updateSize(newContentWidth, newContentHeight);
    }
    _getWidget() {
        return this.widget;
    }
    _isBorderRequired() {
        return false;
    }
    _isPaneEmpty() {
        return super._isPaneEmpty() || !this.hasCaption();
    }
    _isTransparentBackground() {
        return this._isPaneEmpty() && this.visualMode !== 'content';
    }
    _generateInnerBorderClassesUnsafe(element) {
        var classes = super._generateInnerBorderClassesUnsafe(element);
        if (!this._isPaneEmpty()) {
            classes.push(cssDateFilterClassNames.item);
        }
        if (element) {
            if (this._isPaneEmpty()) {
                element.classList.remove(cssDateFilterClassNames.item);
            }
            else {
                element.classList.add(cssDateFilterClassNames.item);
            }
        }
        return classes;
    }
    dispose() {
        if (this.widget) {
            this.widget.dispose();
        }
        super.dispose();
    }
}
dateFilterElement.MinMeasureId = 'DateFilterMin';
dateFilterElement.MaxMeasureId = 'DateFilterMax';
