﻿/**
* DevExpress Dashboard (_clickable-floating-toolbar.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { off, on } from 'devextreme/events';
import { zIndex } from '../../../data/_z-index';
import { FloatingCaptionToolbarBase } from './_floating-toolbar-base';
export class ClickableFloatingCaptionToolbar extends FloatingCaptionToolbarBase {
    static registerToolbar(toolbar) {
        if (ClickableFloatingCaptionToolbar._toolbars.indexOf(toolbar) === -1) {
            ClickableFloatingCaptionToolbar._toolbars.push(toolbar);
        }
    }
    static unregisterToolbar(toolbar) {
        var toolbarIndex = ClickableFloatingCaptionToolbar._toolbars.indexOf(toolbar);
        if (toolbarIndex > -1) {
            ClickableFloatingCaptionToolbar._toolbars.splice(toolbarIndex, 1);
        }
    }
    static activateToolbar(toolbar) {
        ClickableFloatingCaptionToolbar._toolbars
            .filter(t => t !== toolbar)
            .forEach(t => t.hideFloatingPanel());
    }
    constructor(_container, _controlContainer, _popupContainer, encodeHtml, isBottomPosition, itemHasOwnContent) {
        super(_container, _controlContainer, _popupContainer, encodeHtml, isBottomPosition);
        this._itemHasOwnContent = itemHasOwnContent;
    }
    update(options) {
        let heightChanded = super.update(options);
        if (this._getVisibleItems().length > 0) {
            if (this._floatingPanel.isVisible()) {
                this._disableShield();
            }
            else {
                this._enableShield();
            }
        }
        else {
            this._disableShield();
        }
        return heightChanded;
    }
    showFloatingPanel() {
        super.showFloatingPanel();
        this._disableShield();
        ClickableFloatingCaptionToolbar.activateToolbar(this);
    }
    hideFloatingPanel() {
        super.hideFloatingPanel();
        this._enableShield();
    }
    dispose() {
        super.dispose();
        off(this._shieldDiv, 'click.shield');
        ClickableFloatingCaptionToolbar.unregisterToolbar(this);
    }
    _createInstance() {
        return new ClickableFloatingCaptionToolbar(undefined, undefined, undefined, this.encodeHtml, this._isBottomPosition, this._itemHasOwnContent);
    }
    _appendToContainer(toolbarDiv) {
        let floadingPanelDiv = super._appendToContainer(toolbarDiv);
        this._shieldDiv = document.createElement('div');
        this._shieldDiv.style.left = '0';
        this._shieldDiv.style.top = '0';
        this._shieldDiv.style.bottom = '0';
        this._shieldDiv.style.right = '0';
        this._shieldDiv.style.display = 'block';
        if (this._itemHasOwnContent) {
            this._shieldDiv.style.zIndex = (zIndex.floatingToolbar - 1).toString();
        }
        this._shieldDiv.style.position = 'absolute';
        this._subscribeOnShieldEvents();
        this._container.appendChild(this._shieldDiv);
        ClickableFloatingCaptionToolbar.registerToolbar(this);
        return floadingPanelDiv;
    }
    _subscribeOnShieldEvents() {
        on(this._shieldDiv, 'click.shield', () => this.showFloatingPanel());
    }
    _disableShield() {
        this._shieldDiv && (this._shieldDiv.style.display = 'none');
    }
    _enableShield() {
        if (this._getVisibleItems().length > 0) {
            this._shieldDiv && (this._shieldDiv.style.display = 'block');
        }
    }
}
ClickableFloatingCaptionToolbar._toolbars = [];
