﻿/**
* DevExpress Dashboard (_group-item-bindings.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { Disposable, getTemplate } from '@devexpress/analytics-core/analytics-utils';
import * as ko from 'knockout';
import { DashboardItemHeaderHeight } from '../../viewer/_dashboard-item-bindings';
import { SplitterSize } from '../core/_layout-item';
export class GroupViewModel extends Disposable {
    constructor(params) {
        super();
        this.defaultPadding = SplitterSize * 2;
        this.padding = ko.observable(this.defaultPadding);
        this.headerHeight = ko.observable(DashboardItemHeaderHeight);
        this.addDisposable(this.layoutItem = ko.computed(() => ko.unwrap(params.layoutItem)));
        this.addDisposable(ko.computed(() => {
            this.layoutItem().verticalPaddings(this.padding() * 2 + this.headerHeight());
            this.layoutItem().horizontalPaddings(this.padding() * 2);
        }).extend({ deferred: true }));
        this.addDisposable(ko.computed(() => {
            this.layoutItem().setConstraints({
                min: {
                    width: this.layoutItem().items().length === 0 ? 100 : 0,
                    height: this.layoutItem().items().length === 0 ? 100 : 0
                },
                max: {
                    width: Number.MAX_VALUE,
                    height: Number.MAX_VALUE
                }
            });
        }));
    }
}
export const _registerDxDashboardLayoutGroup = () => {
    const componentName = 'dx-dashboard-layout-group';
    if (ko.components.isRegistered(componentName))
        return;
    ko.components.register(componentName, {
        viewModel: GroupViewModel,
        template: getTemplate(componentName)
    });
};
